@@ -3,61 +3,61 @@
 #	Makes the insets for Appendix M(cpt)
 #	[skip srtm which is just a special version of dem2]
 #
-# Use the knowledge that we need 3 pages.
-# 44 original GMT 5 CPTs and the last page has 24 scientific colormaps
-# from Fabio [www.fabiocrameri.ch/visualisation]
+# We have four sets of CPT figures to make:
+# 1a) Our regular, traditional GMT CPTs [44]
+# 1b) The regular Scientific Color Maps* [30]
+# 1c) Categorical CPTs (ours and SCM*)  [18]
+# 1d) Cyclic CPTs from SCM* [5]
+#
+# *from Fabio [www.fabiocrameri.ch/visualisation]
 
 GMT_SHAREDIR=$(gmt --show-sharedir)
 
 cat << EOF > tt.lis
-acton
-bamako
-batlow
-berlin
-bilbao
-broc
-buda
-cork
-davos
-devon
-grayC
-hawaii
-imola
-lajolla
-lapaz
-lisbon
-nuuk
-oleron
-oslo
-roma
-tofino
-tokyo
-turku
-vik
+actonS
+bamakoS
+batlowS
+bilbaoS
+budaS
+categorical
+davosS
+devonS
+grayCS
+hawaiiS
+imolaS
+lajollaS
+lapazS
+nuukS
+osloS
+paired
+tokyoS
+turkuS
 EOF
 
 n=$(cat tt.lis | wc -l)
 let n2=n/2
 let n2=n
 # dy is line spacing and y0 is total box height
-dy=0.75
-y0=$(gmt math -Q $n2 $dy MUL 0.5 MUL =)
+dy=0.6
+y0=$(gmt math -Q $n2 $dy MUL 0.5 MUL 0.1 ADD =)
 
 gmt begin GMT_App_M_1c
+gmt set GMT_THEME cookbook
 gmt set MAP_FRAME_PEN thinner FONT_ANNOT_PRIMARY 8p MAP_TICK_LENGTH_PRIMARY 0.1i MAP_ANNOT_OFFSET_PRIMARY 0.04i
 gmt basemap -R0/6.1/0/$y0 -Jx1i -B0
 
 i=1
-y=0.475
-y2=0.35
+y=0.375
+y2=0.25
 while [ $i -le $n2 ]
 do
-	j=$(expr $i + 1)
-	left=$(sed -n ${j}p tt.lis)
-	right=$(sed -n ${i}p tt.lis)
-	gmt makecpt -H -C$left -T-1/1 > tt.left.cpt
+	j1=$(expr $n2 - $i)
+	j2=$(expr $n2 - $i + 1)
+	left=$(sed -n ${j1}p tt.lis)
+	right=$(sed -n ${j2}p tt.lis)
+	gmt makecpt -H -C$left > tt.left.cpt
 	gmt makecpt -H -C$left -T-1/1/0.25 > tt.left2.cpt
-	gmt makecpt -H -C$right -T-1/1 > tt.right.cpt
+	gmt makecpt -H -C$right > tt.right.cpt
 	gmt makecpt -H -C$right -T-1/1/0.25 > tt.right2.cpt
 	gmt colorbar -Dx1.55i/${y}i+w2.70i/0.125i+h+jTC -Ctt.left.cpt -B0
 	gmt colorbar -Dx4.50i/${y}i+w2.70i/0.125i+h+jTC -Ctt.right.cpt -B0
@@ -81,4 +81,5 @@ do
 	y=$(gmt math -Q $y $dy ADD =)
 	y2=$(gmt math -Q $y2 $dy ADD =)
 done
+rm -f tt.*
 gmt end show
