@@ -1,7 +1,8 @@
 #!/usr/bin/env bash
 gmt begin GMT_Defaults_1b
+	gmt set GMT_THEME cookbook
 	gmt set MAP_FRAME_TYPE plain FORMAT_GEO_MAP ddd:mm:ssF MAP_GRID_CROSS_SIZE_PRIMARY 0i \
-		FONT_ANNOT_PRIMARY +8p MAP_ANNOT_OBLIQUE anywhere
+		FONT_ANNOT_PRIMARY +8p MAP_ANNOT_OBLIQUE anywhere MAP_FRAME_AXES=WESN
 	gmt basemap -X1.5i -R-90/20/-55/25r -JOc-80/25.5/2/69/2.25i -Ba10f5g5
 	gmt text -R0/2.25/0/2 -Jx1i -N -F+f7p,Helvetica-Bold,blue+j << EOF
 -0.15 0.15  RB MAP_ORIGIN_X
