@@ -1,10 +1,11 @@
 #!/usr/bin/env bash
 gmt begin GMT_atan
-	gmt grdgradient -A45 @tut_relief.nc -N -fg -Gtt.t.nc
+	gmt set GMT_THEME cookbook
+	gmt grdgradient -A45 @earth_relief_30s -R-108/-103/35/40 -N -fg -Gtt.t.nc
 	gmt grd2xyz -Z tt.t.nc > tt.d
 	gmt histogram tt.d -R-0.75/0.75/0/20 -JX1.5i/1i -Bx0.5 -By5f5 -BWSne -W0.01 -Gblack -Z1
 
-	gmt text -F+f9p+jLB << EOF
+	gmt text -F+f9p,Helvetica+jLB << EOF
 -0.7	17	Raw
 -0.7	15	slopes
 EOF
@@ -25,7 +26,7 @@ EOF
 	gmt grdgradient -A45 @tut_relief.nc -Nt -fg -Gtt.tt.nc
 	gmt grd2xyz -Z tt.tt.nc > tt.d
 	gmt histogram tt.d -R-0.75/0.75/0/5 -Bx0.5 -By2f1 -BWSne -W0.01  -Gblack -X1.85i -Z1
-	gmt text -F+f9p+jLB << EOF
+	gmt text -F+f9p,Helvetica+jLB << EOF
 -0.7	4.3	tan@+-1@+
 -0.7	3.7	transformed
 EOF
