@@ -1,6 +1,6 @@
 /*--------------------------------------------------------------------
  *
- *	Copyright (c) 1991-2020 by the GMT Team (https://www.generic-mapping-tools.org/team.html)
+ *	Copyright (c) 1991-2021 by the GMT Team (https://www.generic-mapping-tools.org/team.html)
  *	See LICENSE.TXT file for copying and redistribution conditions.
  *
  *	This program is free software; you can redistribute it and/or modify
@@ -20,7 +20,7 @@
  * Version:	6 API
  *
  * Brief synopsis: gmt clear cleans up by removing files or dirs.
- *	gmt clear [all | cache | data[=<planet>] | sessions | settings ]
+ *	gmt clear [all | cache | data[=<planet>] | geography[=<name>] | sessions | settings ]
  */
 
 #include "gmt_dev.h"
@@ -29,7 +29,7 @@
 #define THIS_MODULE_CLASSIC_NAME	"clear"
 #define THIS_MODULE_MODERN_NAME	"clear"
 #define THIS_MODULE_LIB		"core"
-#define THIS_MODULE_PURPOSE	"Delete current default settings, or the cache, data or sessions directories"
+#define THIS_MODULE_PURPOSE	"Delete current default settings, or the cache, data, geography or sessions directories"
 #define THIS_MODULE_KEYS	""
 #define THIS_MODULE_NEEDS	""
 #define THIS_MODULE_OPTIONS	"V"
@@ -37,18 +37,31 @@
 static int usage (struct GMTAPI_CTRL *API, int level) {
 	const char *name = gmt_show_name_and_purpose (API, THIS_MODULE_LIB, THIS_MODULE_CLASSIC_NAME, THIS_MODULE_PURPOSE);
 	if (level == GMT_MODULE_PURPOSE) return (GMT_NOERROR);
-	GMT_Message (API, GMT_TIME_NONE, "usage: %s all|cache|data[=<planet>]|sessions|settings [%s]\n\n", name, GMT_V_OPT);
+	GMT_Usage (API, 0, "usage: %s all|cache|data[=<planet>]|geography[=<name>]|sessions|settings [%s]\n", name, GMT_V_OPT);
 
 	if (level == GMT_SYNOPSIS) return (GMT_MODULE_SYNOPSIS);
 
-	GMT_Message (API, GMT_TIME_NONE, "\tDeletes the specified item.  Choose one of these targets:\n");
-	GMT_Message (API, GMT_TIME_NONE, "\t   cache     Deletes the user\'s cache directory [%s].\n", API->GMT->session.CACHEDIR);
-	GMT_Message (API, GMT_TIME_NONE, "\t   data      Deletes the user\'s data download directory [%s/server].\n", API->GMT->session.USERDIR);
-	GMT_Message (API, GMT_TIME_NONE, "\t             Append =<planet> to limit removal to such data for a specific <planet> [all].\n");
-	GMT_Message (API, GMT_TIME_NONE, "\t   sessions  Deletes the user\'s sessions directory [%s].\n", API->session_dir);
-	GMT_Message (API, GMT_TIME_NONE, "\t   settings  Deletes a modern mode session\'s %s file.\n", GMT_SETTINGS_FILE);
-	GMT_Message (API, GMT_TIME_NONE, "\t   all       All of the above.\n");
-	GMT_Message (API, GMT_TIME_NONE, "\n\tOPTIONS:\n");
+	GMT_Message (API, GMT_TIME_NONE, "  REQUIRED ARGUMENTS:\n");
+	GMT_Usage (API, 1, "\nDeletes the specified item.  Choose one of these targets:");
+	GMT_Usage (API, 2, "cache");
+	GMT_Usage (API, -4, "Deletes the user\'s cache directory:");
+	GMT_Usage (API, -4, "[%s]", API->GMT->session.CACHEDIR);
+	GMT_Usage (API, 2, "data");
+	GMT_Usage (API, -4, "Deletes the user\'s data download directory:");
+	GMT_Usage (API, -4, "[%s/server]", API->GMT->session.USERDIR);
+	GMT_Usage (API, -4, "Append =<planet> to limit removal to such data for a specific <planet> [all].");
+	GMT_Usage (API, 2, "geography");
+	GMT_Usage (API, -4, "Deletes the user\'s geography directory (gshhg, dcw):");
+	GMT_Usage (API, -4, "[%s/geography]", API->GMT->session.USERDIR);
+	GMT_Usage (API, -4, "Append =<name> to limit removal to such data for gshhg or dcw only [all].");
+	GMT_Usage (API, 2, "sessions");
+	GMT_Usage (API, -4, "Deletes the user\'s sessions directory:");
+	GMT_Usage (API, -4, "[%s]", API->session_dir);
+	GMT_Usage (API, 2, "settings");
+	GMT_Usage (API, -4, "Deletes a modern mode session\'s %s file.", GMT_SETTINGS_FILE);
+	GMT_Usage (API, 2, "all");
+	GMT_Usage (API, -4, "All of the above.");
+	GMT_Message (API, GMT_TIME_NONE, "\n  OPTIONAL ARGUMENTS:\n");
 	GMT_Option (API, "V,;");
 
 	return (GMT_MODULE_USAGE);
@@ -112,6 +125,9 @@ static int clear_data (struct GMTAPI_CTRL *API, char *planet) {
 						sprintf (current_d3, "%s/%s/%s/%s", server_dir, dir1[d1], dir2[d2], dir3[d3]);	/* E.g., ~/.gmt/server/earth/earth_relief/earth_relief_15s_p */
 						if (gmt_remove_dir (API, current_d3, false)) {
 							GMT_Report (API, GMT_MSG_ERROR, "Unable to remove directory %s [permissions?]\n", current_d3);
+							gmtlib_free_dir_list (GMT, &dir1);
+							gmtlib_free_dir_list (GMT, &dir2);
+							gmtlib_free_dir_list (GMT, &dir3);
 							return GMT_NOERROR;
 						}
 						d3++;
@@ -120,6 +136,8 @@ static int clear_data (struct GMTAPI_CTRL *API, char *planet) {
 				}
 				if (gmt_remove_dir (API, current_d2, false)) {
 					GMT_Report (API, GMT_MSG_ERROR, "Unable to remove directory %s [permissions?]\n", current_d2);
+					gmtlib_free_dir_list (GMT, &dir1);
+					gmtlib_free_dir_list (GMT, &dir2);
 					return GMT_NOERROR;
 				}
 				d2++;
@@ -128,6 +146,7 @@ static int clear_data (struct GMTAPI_CTRL *API, char *planet) {
 		}
 		if (gmt_remove_dir (API, current_d1, false)) {
 			GMT_Report (API, GMT_MSG_ERROR, "Unable to remove directory %s [permissions?]\n", current_d1);
+			gmtlib_free_dir_list (GMT, &dir1);
 			return GMT_NOERROR;
 		}
 		d1++;
@@ -160,7 +179,7 @@ static int clear_sessions (struct GMTAPI_CTRL *API) {
 #ifdef _WIN32
 	char* t = gmt_strrep(API->session_dir, "/", "\\");		/* rmdir needs paths with back-slashes */
 	strcpy(del_cmd, "rmdir /s /q ");
-	strcat(del_cmd, t);
+	strncat(del_cmd, t, PATH_MAX-1);
 	gmt_M_str_free(t);
 #else
 	sprintf (del_cmd, "rm -rf %s", API->session_dir);
@@ -174,6 +193,47 @@ static int clear_sessions (struct GMTAPI_CTRL *API) {
 	return GMT_NOERROR;
 }
 
+static int clear_geography (struct GMTAPI_CTRL *API, char *data) {
+	int d;
+	char geography_dir[PATH_MAX] = {""}, current_dir[PATH_MAX] = {""};
+	char **dir;
+	struct GMT_CTRL *GMT = API->GMT;
+
+	sprintf (geography_dir, "%s/geography", API->GMT->session.USERDIR);
+	if (access (geography_dir, F_OK)) {
+		GMT_Report (API, GMT_MSG_INFORMATION, "No directory named %s\n", geography_dir);
+		return GMT_FILE_NOT_FOUND;
+	}
+	if ((dir = gmtlib_get_dirs (GMT, geography_dir)) == NULL) {
+		GMT_Report (API, GMT_MSG_NOTICE, "No directories found under %s\n", geography_dir);
+		return GMT_NOERROR;
+	}
+
+	d = 0;
+	while (dir[d]) {	/* Look through geography subdirectories */
+		if (data && strcmp (dir[d], data)) {	/* Not a dataset that we want to delete */
+			d++;
+			continue;
+		}
+		sprintf (current_dir, "%s/%s", geography_dir, dir[d]);	/* E.g., ~/.gmt/geography/gshhg */
+		if (gmt_remove_dir (API, current_dir, false)) {
+			GMT_Report (API, GMT_MSG_ERROR, "Unable to remove directory %s [permissions?]\n", current_dir);
+			gmtlib_free_dir_list (GMT, &dir);
+			return GMT_NOERROR;
+		}
+		d++;
+	}
+	gmtlib_free_dir_list (GMT, &dir);
+
+	if (data == NULL) {
+		if (gmt_remove_dir (API, geography_dir, false)) {
+			GMT_Report (API, GMT_MSG_ERROR, "Unable to remove directory %s [permissions?]\n", geography_dir);
+			return GMT_NOERROR;
+		}
+	}
+	return GMT_NOERROR;
+}
+
 #define bailout(code) {gmt_M_free_options (mode); return (code);}
 #define Return(code) {gmt_end_module (GMT, GMT_cpy); bailout (code);}
 
@@ -208,6 +268,8 @@ EXTERN_MSC int GMT_clear (void *V_API, int mode, void *args) {
 				error = GMT_RUNTIME_ERROR;
 			if (clear_data (API, NULL))
 				error = GMT_RUNTIME_ERROR;
+			if (clear_geography (API, NULL))
+				error = GMT_RUNTIME_ERROR;
 			if (clear_sessions (API))
 				error = GMT_RUNTIME_ERROR;
 			if (API->GMT->current.setting.run_mode == GMT_MODERN && clear_defaults (API))
@@ -227,6 +289,12 @@ EXTERN_MSC int GMT_clear (void *V_API, int mode, void *args) {
 			if (clear_sessions (API))
 				error = GMT_RUNTIME_ERROR;
 		}
+		else if (!strncmp (opt->arg, "geography", 9U)) {	/* Clear the geography dir */
+			char *set = strchr (opt->arg, '=');
+			if (set) set++;	/* Skip past the = sign */
+			if (clear_geography (API, set))
+				error = GMT_RUNTIME_ERROR;
+		}
 		else if (!strcmp (opt->arg, "settings") || !strcmp (opt->arg, "defaults") || !strcmp (opt->arg, "conf")) {	/* Clear the default settings in modern mode */
 			if (API->GMT->current.setting.run_mode == GMT_CLASSIC) {
 				GMT_Report (API, GMT_MSG_ERROR, "Target \"%s\" is only valid in a modern mode session\n", opt->arg);
