@@ -1,7 +1,6 @@
 /*--------------------------------------------------------------------
- *	$Id: gmt_fft.h 19521 2017-12-27 22:38:24Z pwessel $
  *
- *	Copyright (c) 1991-2018 by P. Wessel, W. H. F. Smith, R. Scharroo, J. Luis and F. Wobbe
+ *	Copyright (c) 1991-2021 by the GMT Team (https://www.generic-mapping-tools.org/team.html)
  *	See LICENSE.TXT file for copying and redistribution conditions.
  *
  *	This program is free software; you can redistribute it and/or modify
@@ -13,14 +12,14 @@
  *	MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
  *	GNU Lesser General Public License for more details.
  *
- *	Contact info: gmt.soest.hawaii.edu
+ *	Contact info: www.generic-mapping-tools.org
  *--------------------------------------------------------------------*/
 /*
  *  Structures in support of FFT use.
  *
  * Author:	P. Wessel, derived from W.H.F. Smith implementation
  * Date:	12-FEB-2013
- * Version:	5 API
+ * Version:	6 API
  *
  */
 
@@ -40,20 +39,19 @@ enum GMT_FFT_EXTEND {
 	GMT_FFT_EXTEND_MIRROR_SYMMETRY,
 	GMT_FFT_EXTEND_NONE,
 	GMT_FFT_EXTEND_NOT_SET};
-	
+
 enum GMT_FFT_TREND {
 	GMT_FFT_REMOVE_NOT_SET = -1,
 	GMT_FFT_REMOVE_NOTHING = 0,
 	GMT_FFT_REMOVE_MEAN,
 	GMT_FFT_REMOVE_MID,
 	GMT_FFT_REMOVE_TREND};
-	
+
 enum GMT_FFT_KCHOICE {
 	GMT_FFT_K_IS_KX = 0,
 	GMT_FFT_K_IS_KY,
-	GMT_FFT_K_IS_KR,
-	};
-	
+	GMT_FFT_K_IS_KR};
+
 enum GMT_FFT_DIMSET {
 	GMT_FFT_UNSPECIFIED = 0,
 	GMT_FFT_EXTEND,
