@@ -1,6 +1,6 @@
 /*--------------------------------------------------------------------
  *
- *	Copyright (c) 1991-2020 by the GMT Team (https://www.generic-mapping-tools.org/team.html)
+ *	Copyright (c) 1991-2021 by the GMT Team (https://www.generic-mapping-tools.org/team.html)
  *	See LICENSE.TXT file for copying and redistribution conditions.
  *
  *	This program is free software; you can redistribute it and/or modify
@@ -95,13 +95,14 @@ struct GMT_GDALREAD_IN_CTRL {
 	} P;
 	struct GD_p {	/* Pad array in output */
 		bool active;
-		int pad;
+		unsigned int pad[4];
 	} p;
 	struct GD_W {	/* Convert proj4 string into WKT */
 		bool active;
 	} W;
 	struct GD_R {	/* Sub-region in referenced coords */
 		bool active;
+		bool periodic;
 		char *region;
 	} R;
 	struct GD_Z {	/* Tell to store data in a complex array */
@@ -234,4 +235,11 @@ struct GMT_GDALLIBRARIFIED_CTRL {
 	} M;
 };
 
+struct OGRREAD_CTRL {
+	int    info;            /* If != 0 gmt_ogrread will return only Info (Will it?) */
+	int    layer;           /* If >= 0 will return only data from that layer. Use negative to return all layers */ 
+	char  *name;            /* Vector file name */ 
+	double region[6];       /* For when a sub-region is required */ 
+};
+
 #endif  /* GMT_GDALREAD_H */
