@@ -1,6 +1,6 @@
 /*--------------------------------------------------------------------
  *
- *	Copyright (c) 1991-2020 by the GMT Team (https://www.generic-mapping-tools.org/team.html)
+ *	Copyright (c) 1991-2021 by the GMT Team (https://www.generic-mapping-tools.org/team.html)
  *	See LICENSE.TXT file for copying and redistribution conditions.
  *
  *	This program is free software; you can redistribute it and/or modify
@@ -261,7 +261,7 @@ static inline struct MEMORY_ITEM *gmtmemory_treedelete (struct MEMORY_ITEM *t, v
 }
 
 static inline void gmtmemory_treedestroy (struct MEMORY_ITEM **t) {
-	/* Remves all items from the tree rooted at t. */
+	/* Removes all items from the tree rooted at t. */
 	struct MEMORY_ITEM *x = *t;
 	if (x != NULL) {
 		gmtmemory_treedestroy (&x->l);
@@ -446,7 +446,8 @@ void gmtlib_free_tmp_arrays (struct GMT_CTRL *GMT) {
 	for (col = 0; col < GMT->hidden.mem_cols; col++) {	/* For each column, free an array */
 		gmt_M_free (GMT, GMT->hidden.mem_coord[col]);
 	}
-	gmt_M_free (GMT, GMT->hidden.mem_coord);
+	if (GMT->hidden.mem_coord)
+		gmt_M_free (GMT, GMT->hidden.mem_coord);
 	if (GMT->hidden.mem_txt)
 		gmt_M_free (GMT, GMT->hidden.mem_txt);
 	GMT->hidden.mem_rows = GMT->hidden.mem_cols = 0;
@@ -566,8 +567,8 @@ void *gmt_memory_func (struct GMT_CTRL *GMT, void *prev_addr, size_t nelem, size
 
 void gmt_free_func (struct GMT_CTRL *GMT, void *addr, bool align, const char *where) {
 	if (addr == NULL) {
-#ifndef DEBUG
-		/* report freeing unallocated memory only in level GMT_MSG_DEBUG (-V4) */
+#ifdef DEBUG
+		/* Report attempt at freeing unallocated memory only in level GMT_MSG_DEBUG (-V4) */
 		gmtlib_report_func (GMT, GMT_MSG_DEBUG, where,
 				"tried to free unallocated memory\n");
 #endif
