@@ -1,6 +1,6 @@
 /*--------------------------------------------------------------------
  *
- *	Copyright (c) 1991-2020 by the GMT Team (https://www.generic-mapping-tools.org/team.html)
+ *	Copyright (c) 1991-2021 by the GMT Team (https://www.generic-mapping-tools.org/team.html)
  *	See LICENSE.TXT file for copying and redistribution conditions.
  *
  *	This program is free software; you can redistribute it and/or modify
@@ -22,7 +22,7 @@
  * Version:	6 API
  *
  * Brief synopsis: gmtselect is a general-purpose spatial filter.  Data pass
- * or fail basedon one or more conditions.  Seven conditions may be set:
+ * or fail based on one or more conditions.  Seven conditions may be set:
  *
  *	1. Only data inside a rectangular area may pass
  *	2. Only data within a certain distance from given points may pass
@@ -49,7 +49,7 @@
 #define THIS_MODULE_PURPOSE	"Select data table subsets based on multiple spatial criteria"
 #define THIS_MODULE_KEYS	"<D{,CD(=,FD(,LD(=,>D},GG("
 #define THIS_MODULE_NEEDS	""
-#define THIS_MODULE_OPTIONS "-:>JRVabdefghioqs" GMT_OPT("HMm")
+#define THIS_MODULE_OPTIONS "-:>JRVabdefghioqsw" GMT_OPT("HMm")
 
 #define GMTSELECT_N_CLASSES	(GSHHS_MAX_LEVEL + 1)	/* Number of bands separated by the levels */
 
@@ -87,8 +87,10 @@ struct GMTSELECT_CTRL {	/* All control options for this program (except common a
 	} A;
 	struct GMTSELECT_C {	/* [-C[-|=|+]<dist>/<ptfile>] */
 		bool active;
+		bool point;	/* True if we got a single point */
 		int mode;	/* Form of distance calculation (can be negative) */
 		double dist;	/* Radius of influence for each point */
+		double lon, lat;	/* Coordinates give for a single point */
 		char unit;	/* Unit name */
 		char *file;	/* Name of file with points */
 	} C;
@@ -178,71 +180,71 @@ GMT_LOCAL int gmtselect_compare_x (const void *point_1, const void *point_2) {
 static int usage (struct GMTAPI_CTRL *API, int level) {
 	const char *name = gmt_show_name_and_purpose (API, THIS_MODULE_LIB, THIS_MODULE_CLASSIC_NAME, THIS_MODULE_PURPOSE);
 	if (level == GMT_MODULE_PURPOSE) return (GMT_NOERROR);
-	GMT_Message (API, GMT_TIME_NONE, "usage: %s [<table>] [%s]\n", name, GMT_A_OPT);
-	GMT_Message (API, GMT_TIME_NONE, "\t[-C<ptfile>+d%s] [-D<resolution>][+f] [-E[f][n]] [-F<polygon>] [-G<gridmask>] [%s]\n",
-	             GMT_DIST_OPT, GMT_J_OPT);
-	GMT_Message (API, GMT_TIME_NONE, "\t[-I[cfglrsz] [-L<lfile>+d%s[+p]] [-N<info>] [%s]\n\t[%s] [-Z<min>[/<max>][+c<col>][+a][+i]] [%s] "
-	             "[%s]\n\t[%s] [%s] [%s] [%s]\n\t[%s] [%s]\n\t[%s] [%s] [%s] [%s] [%s]\n\n",
-	             GMT_DIST_OPT, GMT_Rgeo_OPT, GMT_V_OPT, GMT_a_OPT, GMT_b_OPT, GMT_d_OPT, GMT_e_OPT, GMT_f_OPT,
-				 GMT_g_OPT, GMT_h_OPT, GMT_i_OPT, GMT_o_OPT, GMT_q_OPT, GMT_s_OPT, GMT_colon_OPT, GMT_PAR_OPT);
+	GMT_Usage (API, 0, "usage: %s [<table>] [%s] [-C<ptfile>|<lon>/<lat>+d%s] [-D<resolution>[+f]] "
+		"[-E[f][n]] [-F<polygon>] [-G<gridmask>] [%s] [-Icfglrsz] [-L<lfile>+d%s[+p]] [-N<info>] "
+		"[%s] [%s] [-Z<min>[/<max>][+c<col>][+a][+i]] [%s] [%s] [%s] [%s] [%s] [%s] [%s] [%s] "
+		"[%s] [%s] [%s] [%s] [%s] [%s]\n", name, GMT_A_OPT, GMT_DIST_OPT, GMT_J_OPT, GMT_DIST_OPT,
+		GMT_Rgeo_OPT, GMT_V_OPT, GMT_a_OPT, GMT_b_OPT, GMT_d_OPT, GMT_e_OPT, GMT_f_OPT,
+		GMT_g_OPT, GMT_h_OPT, GMT_i_OPT, GMT_o_OPT, GMT_q_OPT, GMT_s_OPT, GMT_w_OPT, GMT_colon_OPT, GMT_PAR_OPT);
 
 	if (level == GMT_SYNOPSIS) return (GMT_MODULE_SYNOPSIS);
 
-	GMT_Message (API, GMT_TIME_NONE, "\tOPTIONS:\n");
+	GMT_Message (API, GMT_TIME_NONE, "  REQUIRED ARGUMENTS:\n");
 	GMT_Option (API, "<");
+	GMT_Message (API, GMT_TIME_NONE, "\n  OPTIONAL ARGUMENTS:\n");
 	gmt_GSHHG_syntax (API->GMT, 'A');
-	GMT_Message (API, GMT_TIME_NONE, "\t   (ignored  unless -N is set).\n");
-	gmt_dist_syntax (API->GMT, 'C', "Pass locations that are within <dist> of any point in the ASCII <ptfile>.");
-	GMT_Message (API, GMT_TIME_NONE, "\t   Give distance as 0 if 3rd column of <ptfile> has individual distances.\n");
-	GMT_Message (API, GMT_TIME_NONE, "\t   Use -R -J to compute mapped Cartesian distances in cm, inch, m, or points [%s].\n",
-	             API->GMT->session.unit_name[API->GMT->current.setting.proj_length_unit]);
-	GMT_Message (API, GMT_TIME_NONE, "\t-D Choose one of the following resolutions: (Ignored unless -N is set).\n");
-	GMT_Message (API, GMT_TIME_NONE, "\t     f - full resolution (may be very slow for large regions).\n");
-	GMT_Message (API, GMT_TIME_NONE, "\t     h - high resolution (may be slow for large regions).\n");
-	GMT_Message (API, GMT_TIME_NONE, "\t     i - intermediate resolution.\n");
-	GMT_Message (API, GMT_TIME_NONE, "\t     l - low resolution [Default].\n");
-	GMT_Message (API, GMT_TIME_NONE, "\t     c - crude resolution, for tasks that need crude continent outlines only.\n");
-	GMT_Message (API, GMT_TIME_NONE, "\t   Append +f to use a lower resolution should the chosen one not be available [abort].\n");
-	GMT_Message (API, GMT_TIME_NONE, "\t-E Indicate if points exactly on a polygon boundary are inside or outside.\n");
-	GMT_Message (API, GMT_TIME_NONE, "\t   Append f and/or n to modify the -F option or -N option, respectively,\n");
-	GMT_Message (API, GMT_TIME_NONE, "\t   to consider such points to be outside the feature [inside].\n");
-	GMT_Message (API, GMT_TIME_NONE, "\t-F Pass locations that are inside the polygons in the ASCII <polygon> file.\n");
-	GMT_Message (API, GMT_TIME_NONE, "\t-G Pass locations that are inside the non-zero, non-NaN nodes of the grid <gridmask>.\n");
-	GMT_Message (API, GMT_TIME_NONE, "\t-I Reverse the tests, i.e., pass locations outside the region.\n");
-	GMT_Message (API, GMT_TIME_NONE, "\t   Supply a combination of cflrz where each flag means:\n");
-	GMT_Message (API, GMT_TIME_NONE, "\t     c will pass locations beyond the minimum distance to the points in -C.\n");
-	GMT_Message (API, GMT_TIME_NONE, "\t     f will pass locations outside the polygons in -F.\n");
-	GMT_Message (API, GMT_TIME_NONE, "\t     g will pass locations outside the grid mask in -G.\n");
-	GMT_Message (API, GMT_TIME_NONE, "\t     l will pass locations beyond the minimum distance to the lines in -L.\n");
-	GMT_Message (API, GMT_TIME_NONE, "\t     r will pass locations outside the region given in -R [and -J].\n");
-	GMT_Message (API, GMT_TIME_NONE, "\t     s will pass locations that otherwise would be skipped in -N.\n");
-	GMT_Message (API, GMT_TIME_NONE, "\t     z will pass locations outside the range given in -Z (single -Z required).\n");
+	GMT_Usage (API, -2, "Note: Ignored  unless -N is set.");
+	gmt_dist_syntax (API->GMT, "C<ptfile>|<lon>/<lat>+d" GMT_DIST_OPT , "Pass locations that are within <dist> of any point in the ASCII <ptfile>.");
+	GMT_Usage (API, -2, "Give distance as 0 if 3rd column of <ptfile> has individual distances. "
+		"For a single point you can instead specify <lon>/<lat>+d[unit]. "
+		"Use -R -J to compute mapped Cartesian distances in cm, inch, m, or points [%s].",
+		API->GMT->session.unit_name[API->GMT->current.setting.proj_length_unit]);
+	gmt_GSHHG_resolution_syntax (API->GMT, 'D', "Ignored unless -N is set.");
+	GMT_Usage (API, 1, "\n-E[f][n]");
+	GMT_Usage (API, -2, "Indicate if points exactly on a polygon boundary are inside [Default] or outside. "
+		"Append f and/or n to modify the -F option or -N option, respectively, "
+		"to consider such points to be outside the feature.");
+	GMT_Usage (API, 1, "\n-F<polygon>");
+	GMT_Usage (API, -2, "Pass locations that are inside the polygons in the ASCII <polygon> file.");
+	GMT_Usage (API, 1, "\n-G<gridmask>");
+	GMT_Usage (API, -2, "Pass locations that are inside the non-zero, non-NaN nodes of the grid <gridmask>.");
+	GMT_Usage (API, 1, "\n-Icfglrsz");
+	GMT_Usage (API, -2, "Reverse the tests, i.e., pass locations outside the selection. "
+		"Supply any combination of cflrz where each flag means:");
+	GMT_Usage (API, 3, "c: Pass locations beyond the minimum distance to the points in -C.");
+	GMT_Usage (API, 3, "f: Pass locations outside the polygons in -F.");
+	GMT_Usage (API, 3, "g: Pass locations outside the grid mask in -G.");
+	GMT_Usage (API, 3, "l: Pass locations beyond the minimum distance to the lines in -L.");
+	GMT_Usage (API, 3, "r: Pass locations outside the region given in -R (and -J).");
+	GMT_Usage (API, 3, "s: Pass locations that otherwise would be skipped in -N.");
+	GMT_Usage (API, 3, "z: Pass locations outside the range given in -Z (single -Z required).");
 	GMT_Option (API, "J");
-	gmt_dist_syntax (API->GMT, 'L', "Pass locations that are within <dist> of any line in ASCII <linefile>.");
-	GMT_Message (API, GMT_TIME_NONE, "\t   Give distance as 0 if 2nd column of segment headers have individual distances.\n");
-	GMT_Message (API, GMT_TIME_NONE, "\t   Use -R -J to compute mapped Cartesian distances in cm, inch, or points [%s].\n",
-	             API->GMT->session.unit_name[API->GMT->current.setting.proj_length_unit]);
-	GMT_Message (API, GMT_TIME_NONE, "\t   Optionally, Append +p to exclude points projecting beyond a line's endpoints.\n");
-	GMT_Message (API, GMT_TIME_NONE, "\t-N Set if a point outside or inside a geographic feature should be s(kipped) or k(ept).\n");
-	GMT_Message (API, GMT_TIME_NONE, "\t   Append o to let feature boundary be considered outside [Default is inside].\n");
-	GMT_Message (API, GMT_TIME_NONE, "\t   Specify this information with s or k using 1 of 2 formats:\n");
-	GMT_Message (API, GMT_TIME_NONE, "\t     -N<wet>/<dry>.\n");
-	GMT_Message (API, GMT_TIME_NONE, "\t     -N<ocean>/<land>/<lake>/<island>/<pond>.\n");
-	GMT_Message (API, GMT_TIME_NONE, "\t   k means keep and s means skip [Default is s/k/s/k/s (i.e., s/k)].\n");
+	gmt_dist_syntax (API->GMT, "L<lfile>+d" GMT_DIST_OPT "[+p]", "Pass locations that are within <dist> of any line in ASCII <lfile>.");
+	GMT_Usage (API, -2, "Give distance as 0 if 2nd column of segment headers have individual distances. "
+		"Use -R -J to compute mapped Cartesian distances in cm, inch, or points [%s]. "
+		"Optionally, Append +p to exclude points projecting beyond a line's endpoints.",
+		API->GMT->session.unit_name[API->GMT->current.setting.proj_length_unit]);
+	GMT_Usage (API, 1, "\n-N<info>");
+	GMT_Usage (API, -2, "Determine if a point outside or inside a geographic feature should be s(kipped) or k(ept). "
+		"Specify this information with s or k using 1 of 2 formats:");
+	GMT_Usage (API, 3, "%s <wet>/<dry>.", GMT_LINE_BULLET);
+	GMT_Usage (API, 3, "%s <ocean>/<land>/<lake>/<island>/<pond>.", GMT_LINE_BULLET);
+	GMT_Usage (API, -2, "[Default is s/k/s/k/s (i.e., s/k)].");
 	GMT_Option (API, "R,V");
-	GMT_Message (API, GMT_TIME_NONE, "\t-Z Assume the 3rd data column contains z-values and we want to keep records with\n");
-	GMT_Message (API, GMT_TIME_NONE, "\t   <min> <= z <= <max>.  Use - for <min> or <max> if there is no lower/upper limit.\n");
-	GMT_Message (API, GMT_TIME_NONE, "\t   Append +c<col> to select another column than the third [2].\n");
-	GMT_Message (API, GMT_TIME_NONE, "\t   If <max> is not given we pass records whose z equal <min> within 5 ulps.\n");
-	GMT_Message (API, GMT_TIME_NONE, "\t   The -Z option is repeatable.  If given more than once then these modifiers may be useful:\n");
-	GMT_Message (API, GMT_TIME_NONE, "\t     +a: Pass the record if any of the tests are true [all tests must be true in order to pass].\n");
-	GMT_Message (API, GMT_TIME_NONE, "\t     +i: Reverse an individual test since -Iz only applies to a single test.\n");
-	GMT_Message (API, GMT_TIME_NONE, "\t     If +c is not given then it is incremented automatically, starting at 2.\n");
+	GMT_Usage (API, 1, "\n-Z<min>[/<max>][+c<col>][+a][+i]");
+	GMT_Usage (API, -2, "Assume the 3rd data column contains z-values and we want to keep records with "
+		"<min> <= z <= <max>.  Use - for <min> or <max> if there is no lower/upper limit. "
+		"If /<max> is not appended then we pass records whose z equal <min> within 5 ulps. "
+		"Append +c<col> to select another column than the third [2]. "
+		"The -Z option is repeatable.  If given more than once then these modifiers may be useful:");
+	GMT_Usage (API, 3, "+a Pass the record if any of the tests are true [all tests must be true in order to pass].");
+	GMT_Usage (API, 3, "+i Reverse an individual test since -Iz only applies to a single test.");
+	GMT_Usage (API, -2, "If +c is not given then it is incremented automatically, starting at 2.");
 	GMT_Option (API, "a,bi0");
-	if (gmt_M_showusage (API)) GMT_Message (API, GMT_TIME_NONE, "\t   Default is 2 input columns (3 if -Z is used).\n");
+	if (gmt_M_showusage (API)) GMT_Usage (API, -2, "Default is 2 input columns (3 if -Z is used).");
 	GMT_Option (API, "bo,d,e,f,g,h,i");
-	if (gmt_M_showusage (API)) GMT_Message (API, GMT_TIME_NONE, "\t   Does not apply to files given via -C, -F, or -L.\n");
-	GMT_Option (API, "o,q,s,:,.");
+	if (gmt_M_showusage (API)) GMT_Usage (API, -2, "Does not apply to files given via -C, -F, or -L.");
+	GMT_Option (API, "o,q,s,w,:,.");
 
 	return (GMT_MODULE_USAGE);
 }
@@ -327,36 +329,50 @@ static int parse (struct GMT_CTRL *GMT, struct GMTSELECT_CTRL *Ctrl, struct GMT_
 		switch (opt->option) {
 
 			case '<':	/* Skip input files */
-				if (GMT_Get_FilePath (GMT->parent, GMT_IS_DATASET, GMT_IN, GMT_FILE_REMOTE, &(opt->arg))) n_errors++;;
+				if (GMT_Get_FilePath (API, GMT_IS_DATASET, GMT_IN, GMT_FILE_REMOTE, &(opt->arg))) n_errors++;;
 				break;
 
 			/* Processes program-specific parameters */
 
 			case 'A':	/* Limit GSHHS features */
+				n_errors += gmt_M_repeated_module_option (API, Ctrl->A.active);
 				Ctrl->A.active = true;
 				n_errors += gmt_set_levels (GMT, opt->arg, &Ctrl->A.info);
 				break;
-			case 'C':	/* Near a point test  Syntax -C<pfile>+d<distance>  */
+			case 'C':	/* Near a point test  Syntax -C<pfile>+d<distance> or -C<lon/lat>+d<distance> */
+				n_errors += gmt_M_repeated_module_option (API, Ctrl->C.active);
 				Ctrl->C.active = true;
 				if ((c = strstr (opt->arg, "+d")) == NULL) {	/* Must be old syntax or error */
 					n_errors += gmtselect_old_C_parser (API, opt->arg, Ctrl);
 					break;
 				}
-				/* Here we perform new syntax parsing */
+				/* Here we perform new syntax parsing, and c points to +d<arg> */
 				if (opt->arg[0] == 0) {
 					GMT_Report (API, GMT_MSG_ERROR, "Option -C: No file given\n");
 					n_errors++;
 				}
-				else
-					Ctrl->C.file = gmt_get_filename (API, opt->arg, "d");
-				Ctrl->C.mode = gmt_get_distance (GMT, &c[2], &(Ctrl->C.dist), &(Ctrl->C.unit));
+				else {
+					Ctrl->C.mode = gmt_get_distance (GMT, &c[2], &(Ctrl->C.dist), &(Ctrl->C.unit));
+					c[0] = '\0';	/* Chop off module-specific modifier */
+					Ctrl->C.file = gmt_get_filename (API, opt->arg, NULL);
+					if (gmt_count_char (GMT, Ctrl->C.file, '/') == 1 && gmt_access (GMT, Ctrl->C.file, R_OK)) {	/* Check if we got a lon/lat point */
+						if ((j = sscanf (Ctrl->C.file, "%[^/]/%s", za, zb)) != 2) continue;	/* No, strange... */
+						n_errors += gmt_verify_expectations (GMT, gmt_M_type (GMT, GMT_IN, GMT_X), gmt_scanf_arg (GMT, za, gmt_M_type (GMT, GMT_IN, GMT_X), false, &Ctrl->C.lon), za);
+						n_errors += gmt_verify_expectations (GMT, gmt_M_type (GMT, GMT_IN, GMT_Y), gmt_scanf_arg (GMT, zb, gmt_M_type (GMT, GMT_IN, GMT_Y), false, &Ctrl->C.lat), zb);
+						if (n_errors == 0)
+							Ctrl->C.point = true;
+					}
+					c[0] = '+';	/* Restore module-specific modifier */
+				}
 				break;
 			case 'D':	/* Set GSHHS resolution */
+				n_errors += gmt_M_repeated_module_option (API, Ctrl->D.active);
 				Ctrl->D.active = true;
 				Ctrl->D.set = opt->arg[0];
 				Ctrl->D.force = (opt->arg[1] == '+' && (opt->arg[2] == 'f' || opt->arg[2] == '\0'));
 				break;
 			case 'E':	/* On-boundary selection */
+				n_errors += gmt_M_repeated_module_option (API, Ctrl->E.active);
 				Ctrl->E.active = true;
 				for (j = 0; opt->arg[j]; j++) {
 					switch (opt->arg[j]) {
@@ -370,16 +386,19 @@ static int parse (struct GMT_CTRL *GMT, struct GMTSELECT_CTRL *Ctrl, struct GMT_
 				}
 				break;
 			case 'F':	/* Inside/outside polygon test */
+				n_errors += gmt_M_repeated_module_option (API, Ctrl->F.active);
 				Ctrl->F.active = true;
 				if (opt->arg[0]) Ctrl->F.file = strdup (opt->arg);
-				if (GMT_Get_FilePath (GMT->parent, GMT_IS_DATASET, GMT_IN, GMT_FILE_REMOTE, &(Ctrl->F.file))) n_errors++;
+				if (GMT_Get_FilePath (API, GMT_IS_DATASET, GMT_IN, GMT_FILE_REMOTE, &(Ctrl->F.file))) n_errors++;
 				break;
 			case 'G':	/* In-grid selection */
+				n_errors += gmt_M_repeated_module_option (API, Ctrl->G.active);
 				Ctrl->G.active = true;
 				if (opt->arg[0]) Ctrl->G.file = strdup (opt->arg);
-				if (GMT_Get_FilePath (GMT->parent, GMT_IS_GRID, GMT_IN, GMT_FILE_REMOTE, &(Ctrl->G.file))) n_errors++;
+				if (GMT_Get_FilePath (API, GMT_IS_GRID, GMT_IN, GMT_FILE_REMOTE, &(Ctrl->G.file))) n_errors++;
 				break;
 			case 'I':	/* Invert these tests */
+				n_errors += gmt_M_repeated_module_option (API, Ctrl->I.active);
 				Ctrl->I.active = true;
 				for (j = 0; opt->arg[j]; j++) {
 					switch (opt->arg[j]) {
@@ -398,26 +417,31 @@ static int parse (struct GMT_CTRL *GMT, struct GMTSELECT_CTRL *Ctrl, struct GMT_
 				}
 				break;
 			case 'L':	/* Near a line test -L<lfile>+d%s[+p]] */
+				n_errors += gmt_M_repeated_module_option (API, Ctrl->L.active);
 				Ctrl->L.active = true;
 				if ((c = strstr (opt->arg, "+d")) == NULL) {	/* Must be old syntax or error */
 					n_errors += gmtselect_old_L_parser (API, opt->arg, Ctrl);
 					break;
 				}
-				/* Here we perform new syntax parsing */
+				/* Here we perform new syntax parsing, withg c pointing to the +d<arg> string */
 				if (gmt_validate_modifiers (GMT, opt->arg, 'L', "dp", GMT_MSG_ERROR)) n_errors++;
 				if (opt->arg[0] == 0) {
 					GMT_Report (API, GMT_MSG_ERROR, "Option -L: No file given\n");
 					n_errors++;
 				}
-				else
-					Ctrl->L.file = gmt_get_filename (API, opt->arg, "dp");
-				if (gmt_get_modifier (opt->arg, 'd', buffer)) {
-					Ctrl->L.mode = gmt_get_distance (GMT, buffer, &(Ctrl->L.dist), &(Ctrl->L.unit));
+				else {	/* Use c to explore modifiers */
+					if (gmt_get_modifier (c, 'd', buffer)) {
+						Ctrl->L.mode = gmt_get_distance (GMT, buffer, &(Ctrl->L.dist), &(Ctrl->L.unit));
+					}
+					if (gmt_get_modifier (c, 'p', buffer))
+						Ctrl->L.end_mode = 10;
+					c[0] = '\0';	/* Chop off module-specific modifier */
+					Ctrl->L.file = gmt_get_filename (API, opt->arg, NULL);
+					c[0] = '+';	/* Restore module-specific modifier */
 				}
-				if (gmt_get_modifier (opt->arg, 'p', buffer))
-					Ctrl->L.end_mode = 10;
 				break;
 			case 'N':	/* Inside/outside GSHHS land */
+				n_errors += gmt_M_repeated_module_option (API, Ctrl->N.active);
 				Ctrl->N.active = true;
 				strncpy (buffer, opt->arg, GMT_BUFSIZ);
 				if (buffer[strlen(buffer)-1] == 'o' && gmt_M_compat_check (GMT, 4)) { /* Edge is considered outside */
@@ -507,7 +531,7 @@ static int parse (struct GMT_CTRL *GMT, struct GMTSELECT_CTRL *Ctrl, struct GMT_
 	n_errors += gmt_M_check_condition (GMT, Ctrl->C.mode == -1, "Option -C: Unrecognized distance unit\n");
 	n_errors += gmt_M_check_condition (GMT, Ctrl->C.mode == -2, "Option -C: Unable to decode distance\n");
 	n_errors += gmt_M_check_condition (GMT, Ctrl->C.mode == -3, "Option -C: Distance is negative\n");
-	n_errors += gmt_M_check_condition (GMT, Ctrl->C.active && gmt_access (GMT, Ctrl->C.file, R_OK),
+	n_errors += gmt_M_check_condition (GMT, Ctrl->C.active && !Ctrl->C.point && gmt_access (GMT, Ctrl->C.file, R_OK),
 	                                   "Option -C: Cannot read file %s!\n", Ctrl->C.file);
 	n_errors += gmt_M_check_condition (GMT, Ctrl->F.active && gmt_access (GMT, Ctrl->F.file, R_OK),
 	                                   "Option -F: Cannot read file %s!\n", Ctrl->F.file);
@@ -594,7 +618,7 @@ EXTERN_MSC int GMT_gmtselect (void *V_API, int mode, void *args) {
 		else
 			do_project = true;	/* Only true when the USER selected -J, not when we supply a dummy -Jx1d */
 		Ctrl->dbg.step = 0.01;
-		if (gmt_M_is_geographic (GMT, GMT_IN)) {
+		if (gmt_M_x_is_lon (GMT, GMT_IN)) {
 			while (GMT->common.R.wesn[XLO] < 0.0 && GMT->common.R.wesn[XHI] < 0.0) {	/* Make all-negative longitude range positive instead */
 				GMT->common.R.wesn[XLO] += 360.0;
 				GMT->common.R.wesn[XHI] += 360.0;
@@ -607,7 +631,7 @@ EXTERN_MSC int GMT_gmtselect (void *V_API, int mode, void *args) {
 	if (do_project) GMT_Report (API, GMT_MSG_WARNING, "-J means all data will be projected before tests are applied\n");
 
 	if (Ctrl->N.active) {	/* Set up GSHHS */
-		if (Ctrl->D.force) Ctrl->D.set = gmt_shore_adjust_res (GMT, Ctrl->D.set);
+		if (Ctrl->D.force) Ctrl->D.set = gmt_shore_adjust_res (GMT, Ctrl->D.set, true);
 		if (Ctrl->D.active) base = gmt_set_resolution (GMT, &Ctrl->D.set, 'D');
 		if (Ctrl->N.mode) {	/* Post-process -N choice */
 			Ctrl->N.mask[3] = Ctrl->N.mask[1];
@@ -638,19 +662,30 @@ EXTERN_MSC int GMT_gmtselect (void *V_API, int mode, void *args) {
 
 	if (error == GMT_NOT_A_VALID_TYPE) Return (error);
 
-	gmt_disable_bghi_opts (GMT);	/* Do not want any -b -g -h -i to affect the reading from -C,-F,-L files */
+	gmt_disable_bghio_opts (GMT);	/* Do not want any -b -g -h -i -o to affect the reading from -C,-F,-L files */
 
-	if (Ctrl->C.active) { 	/* Initialize point structure used in test for proximity to points [use Ctrl->C.dist ]*/
-		if ((Cin = GMT_Read_Data (API, GMT_IS_DATASET, GMT_IS_FILE, GMT_IS_POINT, GMT_IO_ASCII, NULL, Ctrl->C.file, NULL)) == NULL) {
-			Return (API->error);
+	if (Ctrl->C.active) { 	/* Initialize point structure used in test for proximity to points [use Ctrl->C.dist] */
+		if (Ctrl->C.point) {
+			uint64_t dim[4] = {1, 1, 1, 2};
+			if ((Cin = GMT_Create_Data (API, GMT_IS_DATASET, GMT_IS_POINT, 0, dim, NULL, NULL, 0, 0, NULL)) == NULL) {
+				Return (API->error);
+			}
+			/* Place the given point in the array */
+			Cin->table[0]->segment[0]->data[GMT_X][0] = Ctrl->C.lon;
+			Cin->table[0]->segment[0]->data[GMT_Y][0] = Ctrl->C.lat;
 		}
-		if (Cin->n_columns < 2) {	/* Trouble */
-			GMT_Report (API, GMT_MSG_ERROR, "Option -C: %s does not have at least 2 columns with coordinates\n", Ctrl->C.file);
-			Return (GMT_RUNTIME_ERROR);
-		}
-		if (Ctrl->C.dist == 0.0 && Cin->n_columns <= 2) {	/* Trouble */
-			GMT_Report (API, GMT_MSG_ERROR, "Option -C: %s does not have a 3rd column with distances, yet -C0/<file> was given\n", Ctrl->C.file);
-			Return (GMT_RUNTIME_ERROR);
+		else {	/* Read from file */
+			if ((Cin = GMT_Read_Data (API, GMT_IS_DATASET, GMT_IS_FILE, GMT_IS_POINT, GMT_IO_ASCII, NULL, Ctrl->C.file, NULL)) == NULL) {
+				Return (API->error);
+			}
+			if (Cin->n_columns < 2) {	/* Trouble */
+				GMT_Report (API, GMT_MSG_ERROR, "Option -C: %s does not have at least 2 columns with coordinates\n", Ctrl->C.file);
+				Return (GMT_RUNTIME_ERROR);
+			}
+			if (Ctrl->C.dist == 0.0 && Cin->n_columns <= 2) {	/* Trouble */
+				GMT_Report (API, GMT_MSG_ERROR, "Option -C: %s does not have a 3rd column with distances, yet -C0/<file> was given\n", Ctrl->C.file);
+				Return (GMT_RUNTIME_ERROR);
+			}
 		}
 		point = Cin->table[0];	/* Can only be one table since we read a single file */
 		for (seg = 0; seg < point->n_segments; seg++) {
@@ -747,7 +782,7 @@ EXTERN_MSC int GMT_gmtselect (void *V_API, int mode, void *args) {
 		}
 	}
 
-	gmt_reenable_bghi_opts (GMT);	/* Recover settings provided by user (if -b -g -h -i were used at all) */
+	gmt_reenable_bghio_opts (GMT);	/* Recover settings provided by user (if -b -g -h -i were used at all) */
 
 	/* Specify input and output expected columns */
 	if ((error = GMT_Set_Columns (API, GMT_IN, 0, GMT_COL_FIX)) != GMT_NOERROR) Return (error);
