@@ -1,6 +1,6 @@
 /*--------------------------------------------------------------------
  *
- *	Copyright (c) 1991-2020 by the GMT Team (https://www.generic-mapping-tools.org/team.html)
+ *	Copyright (c) 1991-2021 by the GMT Team (https://www.generic-mapping-tools.org/team.html)
  *	See LICENSE.TXT file for copying and redistribution conditions.
  *
  *	This program is free software; you can redistribute it and/or modify
@@ -79,6 +79,7 @@ struct GRDGRADIENT_CTRL {
 		/* Note: If -Qc is set with -N then -G is not required. GMT_Encode_Options turns off the primary output */
 		bool active;
 		unsigned int mode;
+		char *file;
 	} Q;
 	struct GRDGRADIENT_S {	/* -S<slopefile> */
 		bool active;
@@ -130,50 +131,61 @@ GMT_LOCAL double grdgradient_specular (double n_columns, double n_rows, double n
 static int usage (struct GMTAPI_CTRL *API, int level) {
 	const char *name = gmt_show_name_and_purpose (API, THIS_MODULE_LIB, THIS_MODULE_CLASSIC_NAME, THIS_MODULE_PURPOSE);
 	if (level == GMT_MODULE_PURPOSE) return (GMT_NOERROR);
-	GMT_Message (API, GMT_TIME_NONE, "usage: %s <ingrid> -G<outgrid> [-A<azim>[/<azim2>]] [-D[a][c][o][n]]\n", name);
-	GMT_Message (API, GMT_TIME_NONE, "\t[-E[s|p|m]<azim>/<elev>[+a<ambient>][+d<diffuse>][+p<specular>][+s<shine>]]\n");
-	GMT_Message (API, GMT_TIME_NONE, "\t[-N[t|e][<amp>][+a<ambient>][+s<sigma>][+o<offset>]] [-Qc|r|R] [%s]\n\t[-S<slopegrid>] [%s] [-fg] [%s] [%s]\n\n", GMT_Rgeo_OPT, GMT_V_OPT, GMT_n_OPT, GMT_PAR_OPT);
+	GMT_Usage (API, 0, "usage: %s %s -G%s [-A<azim>[/<azim2>]] [-D[a][c][o][n]] "
+		"[-E[s|p|m]<azim>/<elev>[+a<ambient>][+d<diffuse>][+p<specular>][+s<shine>]] "
+		"[-N[t|e][<amp>][+a<ambient>][+s<sigma>][+o<offset>]] [-Qc|r|R[+f<file>]] [%s] [-S<slopegrid>] [%s] "
+		"[-fg] [%s] [%s]\n", name, GMT_INGRID, GMT_OUTGRID, GMT_Rgeo_OPT, GMT_V_OPT, GMT_n_OPT, GMT_PAR_OPT);
 
 	if (level == GMT_SYNOPSIS) return (GMT_MODULE_SYNOPSIS);
 
-	GMT_Message (API, GMT_TIME_NONE, "\t<ingrid> is name of input grid file.\n");
-	GMT_Message (API, GMT_TIME_NONE, "\n\tOPTIONS:\n");
-	GMT_Message (API, GMT_TIME_NONE, "\t-A Set azimuth (0-360 CW from North (+y)) for directional derivatives.\n");
-	GMT_Message (API, GMT_TIME_NONE, "\t  -A<azim>/<azim2> will compute two directions and select the one larger in magnitude.\n");
-	GMT_Message (API, GMT_TIME_NONE, "\t  If <azim> is a grid we expect variable azimuths on a grid coregistered with <ingrid>.\n");
-	GMT_Message (API, GMT_TIME_NONE, "\t-D Find the direction of the vector grad z (up-slope direction).\n");
-	GMT_Message (API, GMT_TIME_NONE, "\t   Append a to get the aspect instead (down-slope direction).\n");
-	GMT_Message (API, GMT_TIME_NONE, "\t   Append c to report Cartesian angle (0-360 CCW from East (+x-axis)) [Default: azimuth].\n");
-	GMT_Message (API, GMT_TIME_NONE, "\t   Append o to get bidirectional orientations [0-180] rather than directions [0-360].\n");
-	GMT_Message (API, GMT_TIME_NONE, "\t   Append n to add 90 degrees to the values from c or o.\n");
-	GMT_Message (API, GMT_TIME_NONE, "\t-E Compute Lambertian radiance appropriate to use with grdimage/grdview.\n");
-	GMT_Message (API, GMT_TIME_NONE, "\t   Provide the azimuth and elevation of the light vector.\n");
-	GMT_Message (API, GMT_TIME_NONE, "\t   Append any of [+a<ambient>][+d<diffuse>][+p<specular>][+s<shine>]\n");
-	GMT_Message (API, GMT_TIME_NONE, "\t     for parameters that control the reflectance properties of the surface\n");
-	GMT_Message (API, GMT_TIME_NONE, "\t     [Default values are: +a0.55+d0.6p0.4+s10].\n");
-	GMT_Message (API, GMT_TIME_NONE, "\t   Use -Es for a simpler Lambertian algorithm (note that with this algorithm\n");
-	GMT_Message (API, GMT_TIME_NONE, "\t     you only have to provide azimuth and elevation).\n");
-	GMT_Message (API, GMT_TIME_NONE, "\t   Use -Ep for the Peucker piecewise linear approximation (simpler but faster algorithm).\n");
-	GMT_Message (API, GMT_TIME_NONE, "\t   Use -Em for another algorithm that gives results close to ESRI's 'hillshade' but faster\n");
-	GMT_Message (API, GMT_TIME_NONE, "\t     In this case the azimuth and elevation are hardwired to 315 and 45 degrees;\n");
-	GMT_Message (API, GMT_TIME_NONE, "\t     if you provide other values they will be ignored.\n");
-	GMT_Message (API, GMT_TIME_NONE, "\t-G Output file for results from -A or -D.\n");
-	GMT_Message (API, GMT_TIME_NONE, "\t-N Normalize gradients so that max |grad| = <amp> [1.0].\n");
-	GMT_Message (API, GMT_TIME_NONE, "\t  -Nt will make atan transform, then scale to <amp> [1.0].\n");
-	GMT_Message (API, GMT_TIME_NONE, "\t  -Ne will make exp  transform, then scale to <amp> [1.0].\n");
-	GMT_Message (API, GMT_TIME_NONE, "\t    Append +a<ambient> to add <ambient> to the result [0].\n");
-	GMT_Message (API, GMT_TIME_NONE, "\t    For -Nt|e, optionally append +s<sigma> and/or +o<offset> to set\n");
-	GMT_Message (API, GMT_TIME_NONE, "\t    sigma and offset for the transform [Default estimates from the data].\n");
-	GMT_Message (API, GMT_TIME_NONE, "\t    See -Q to use the same offset, sigma for multiple grid calculations.\n");
-	GMT_Message (API, GMT_TIME_NONE, "\t-Q Control handling of -N arguments from previous calculations. Append code:\n");
-	GMT_Message (API, GMT_TIME_NONE, "\t     c: Create stat file and write the offset and sigma of this run.\n");
-	GMT_Message (API, GMT_TIME_NONE, "\t     r: Read offset and sigma of the previous run from stat file.\n");
-	GMT_Message (API, GMT_TIME_NONE, "\t     R: Remove & read.  As r but also removes the stat file after use.\n");
-	GMT_Message (API, GMT_TIME_NONE, "\t   The values obtained are used if +o and/or +s modifiers in -N are given no argument.\n");
+	GMT_Message (API, GMT_TIME_NONE, "  REQUIRED ARGUMENTS:\n");
+	gmt_ingrid_syntax (API, 0, "Name of input grid");
+	gmt_outgrid_syntax (API, 'G', "Set output grid file for results from -A or -D");
+	GMT_Message (API, GMT_TIME_NONE, "\n  OPTIONAL ARGUMENTS:\n");
+	GMT_Usage (API, 1, "\n-A<azim>[/<azim2>]");
+	GMT_Usage (API, -2, "Set azimuth (0-360 CW from North (+y)) for directional derivatives. "
+		"Appending /<azim2> will compute two directions and select the one larger in magnitude. "
+		"If <azim> is a grid we expect variable azimuths on a grid co-registered with <ingrid>.");
+	GMT_Usage (API, 1, "\n-D[a][c][o][n]");
+	GMT_Usage (API, -2, "Find the direction of the vector grad z (up-slope direction) given directive:");
+	GMT_Usage (API, 3, "a: Get the aspect instead (down-slope direction).");
+	GMT_Usage (API, 3, "c: Report Cartesian angle (0-360 CCW from East (+x-axis)) [Default: azimuth].");
+	GMT_Usage (API, 3, "o: Get bidirectional orientations [0-180] rather than directions [0-360].");
+	GMT_Usage (API, -2, "Append n to add 90 degrees to the values from c or o.");
+	GMT_Usage (API, 1, "\n-E[s|p|m]<azim>/<elev>[+a<ambient>][+d<diffuse>][+p<specular>][+s<shine>]");
+	GMT_Usage (API, -2, "Compute Lambertian radiance appropriate to use with grdimage|grdview. "
+		"Provide the azimuth and elevation of the light vector. "
+		"Optionally, append modifiers to further control the reflectance properties of the surface:");
+	GMT_Usage (API, 3, "+a Set the <ambient> light value [0.55].");
+	GMT_Usage (API, 3, "+d Set the <diffuse> light value [0.6].");
+	GMT_Usage (API, 3, "+p Set <specular> value [0.4].");
+	GMT_Usage (API, 3, "+s Set <shine> value [10].");
+	GMT_Usage (API, -2, "Directives s|p|m can simplify the setup:");
+	GMT_Usage (API, 3, "s: Simpler Lambertian algorithm (only requires azimuth and elevation).");
+	GMT_Usage (API, 3, "p: Peucker piecewise linear approximation (simpler but faster algorithm).");
+	GMT_Usage (API, 3, "m: Similar to ESRI's 'hillshade' but faster. Note: azimuth and elevation are hardwired to 315 and 45 degrees; "
+		"if you provide other values they will be ignored.");
+	GMT_Usage (API, 1, "\n-N[t|e][<amp>][+a<ambient>][+s<sigma>][+o<offset>]");
+	GMT_Usage (API, -2, "Normalize gradients so that max |grad| = <amp> [1.0]. Optional directives:");
+	GMT_Usage (API, 3, "t: Make atan transform, then scale to <amp> [1.0].");
+	GMT_Usage (API, 3, "e: Make exp  transform, then scale to <amp> [1.0].");
+	GMT_Usage (API, -2, "Several modifiers can be used to affect the final result:");
+	GMT_Usage (API, 3, "+a Add <ambient> light [0].");
+	GMT_Usage (API, 3, "+o Offset gradients by <offset> [Default estimates <offset> from the data].");
+	GMT_Usage (API, 3, "+s Normalize gradients by <sigma> [Default estimates <sigma> from the data].");
+	GMT_Usage (API, -2, "See -Q to use the same offset, sigma for multiple grid calculations.");
+	GMT_Usage (API, 1, "\n-Qc|r|R[+f<file>]");
+	GMT_Usage (API, -2, "Control handling of -N arguments from previous calculations. Append directive:");
+	GMT_Usage (API, 3, "c: Create statistics file and write the offset and sigma of this run.");
+	GMT_Usage (API, 3, "r: Read offset and sigma of the previous run from statistics file.");
+	GMT_Usage (API, 3, "R: Remove and read.  As r but also removes the statistics file after use.");
+	GMT_Usage (API, -2, "Optionally, append +f<file> to write/read the statistics to/from a specified file. Note: The values "
+		"obtained are used if +o and/or +s modifiers in -N are given no argument.");
 	GMT_Option (API, "R");
-	GMT_Message (API, GMT_TIME_NONE, "\t-S Output file for |grad z|; requires -D.\n");
+	GMT_Usage (API, 1, "\n-S<slopegrid>");
+	GMT_Usage (API, -2, "Output file for |grad z|; requires -D.");
 	GMT_Option (API, "V");
-	GMT_Message (API, GMT_TIME_NONE, "\t-fg Convert geographic grids to meters using a \"Flat Earth\" approximation.\n");
+	GMT_Usage (API, 1, "\n-fg Convert geographic grids to meters using a \"Flat Earth\" approximation.");
 	GMT_Option (API, "n,.");
 
 	return (GMT_MODULE_USAGE);
@@ -199,12 +211,13 @@ static int parse (struct GMT_CTRL *GMT, struct GRDGRADIENT_CTRL *Ctrl, struct GM
 				if (n_files++ > 0) {n_errors++; continue; }
 				Ctrl->In.active = true;
 				if (opt->arg[0]) Ctrl->In.file = strdup (opt->arg);
-				if (GMT_Get_FilePath (GMT->parent, GMT_IS_GRID, GMT_IN, GMT_FILE_REMOTE, &(Ctrl->In.file))) n_errors++;
+				if (GMT_Get_FilePath (API, GMT_IS_GRID, GMT_IN, GMT_FILE_REMOTE, &(Ctrl->In.file))) n_errors++;
 				break;
 
 			/* Processes program-specific parameters */
 
 			case 'A':	/* Set azimuth(s) */
+				n_errors += gmt_M_repeated_module_option (API, Ctrl->A.active);
 				Ctrl->A.active = true;
 				if (!gmt_access (GMT, opt->arg, F_OK)) {	/* file with variable azimuths exists */
 					Ctrl->A.file = strdup (opt->arg);
@@ -217,6 +230,7 @@ static int parse (struct GMT_CTRL *GMT, struct GRDGRADIENT_CTRL *Ctrl, struct GM
 				}
 				break;
 			case 'D':	/* Find direction of grad|z| */
+				n_errors += gmt_M_repeated_module_option (API, Ctrl->D.active);
 				Ctrl->D.active = true;
 				j = 0;
 				while (opt->arg[j]) {
@@ -234,6 +248,7 @@ static int parse (struct GMT_CTRL *GMT, struct GRDGRADIENT_CTRL *Ctrl, struct GM
 				}
 				break;
 			case 'E':	/* Lambertian family radiance */
+				n_errors += gmt_M_repeated_module_option (API, Ctrl->E.active);
 				Ctrl->E.active = true;
 				switch (opt->arg[0]) {
 					case 'p':	/* Peucker */
@@ -252,6 +267,8 @@ static int parse (struct GMT_CTRL *GMT, struct GRDGRADIENT_CTRL *Ctrl, struct GM
 						}
 						else if (j == 1)
 							Ctrl->E.elevation = 45;
+
+						Ctrl->E.azimuth = -Ctrl->E.azimuth + 90;	/* The algorithm use mat angles but we want azimuths */
 						break;
 					default:
 						Ctrl->E.mode = 3;	/* "full" Lambertian case */
@@ -296,9 +313,10 @@ static int parse (struct GMT_CTRL *GMT, struct GRDGRADIENT_CTRL *Ctrl, struct GM
 				}
 				break;
 			case 'G':	/* Output grid */
+				n_errors += gmt_M_repeated_module_option (API, Ctrl->G.active);
 				Ctrl->G.active = true;
 				if (opt->arg[0]) Ctrl->G.file = strdup (opt->arg);
-				if (GMT_Get_FilePath (GMT->parent, GMT_IS_GRID, GMT_OUT, GMT_FILE_LOCAL, &(Ctrl->G.file))) n_errors++;
+				if (GMT_Get_FilePath (API, GMT_IS_GRID, GMT_OUT, GMT_FILE_LOCAL, &(Ctrl->G.file))) n_errors++;
 				break;
 			case 'L':	/* GMT4 BCs */
 				if (gmt_M_compat_check (GMT, 4)) {
@@ -325,6 +343,7 @@ static int parse (struct GMT_CTRL *GMT, struct GRDGRADIENT_CTRL *Ctrl, struct GM
 					n_errors += gmt_default_error (GMT, opt->option);
 				break;
 			case 'N':	/* Normalization */
+				n_errors += gmt_M_repeated_module_option (API, Ctrl->N.active);
 				Ctrl->N.active = true;	j = 0;
 				if (opt->arg[0] && strchr ("et", opt->arg[0])) {	/* Got -Ne or -Nt, possibly with arguments */
 					Ctrl->N.mode = (opt->arg[0] == 't') ? 1 : 2;
@@ -355,6 +374,11 @@ static int parse (struct GMT_CTRL *GMT, struct GRDGRADIENT_CTRL *Ctrl, struct GM
 				}
 				break;
 			case 'Q':	/* Read/write normalization values */
+				n_errors += gmt_M_repeated_module_option (API, Ctrl->Q.active);
+				if (gmt_validate_modifiers (GMT, opt->arg, 'Q', "f", GMT_MSG_ERROR)) n_errors++;
+				if (gmt_get_modifier (opt->arg, 'f', p)) {
+					if (p[0]) Ctrl->Q.file = strdup (p);
+				}
 				Ctrl->Q.active = true;
 				switch (opt->arg[0]) {
 					case 'r': Ctrl->Q.mode = 1; break;
@@ -367,9 +391,10 @@ static int parse (struct GMT_CTRL *GMT, struct GRDGRADIENT_CTRL *Ctrl, struct GM
 				}
 				break;
 			case 'S':	/* Slope grid */
+				n_errors += gmt_M_repeated_module_option (API, Ctrl->S.active);
 				Ctrl->S.active = true;
 				if (opt->arg[0]) Ctrl->S.file = strdup (opt->arg);
-				if (GMT_Get_FilePath (GMT->parent, GMT_IS_GRID, GMT_OUT, GMT_FILE_LOCAL, &(Ctrl->S.file))) n_errors++;
+				if (GMT_Get_FilePath (API, GMT_IS_GRID, GMT_OUT, GMT_FILE_LOCAL, &(Ctrl->S.file))) n_errors++;
 				break;
 
 			default:	/* Report bad options */
@@ -402,8 +427,8 @@ static int parse (struct GMT_CTRL *GMT, struct GRDGRADIENT_CTRL *Ctrl, struct GM
 EXTERN_MSC int GMT_grdgradient (void *V_API, int mode, void *args) {
 	bool bad, new_grid = false, separate = false;
 	int p[4], mx, error = 0;
-	unsigned int row, col, n;
-	uint64_t ij, ij0, index, n_used = 0;
+	unsigned int row, col, n, orig_pad[4];
+	uint64_t ij, ij0, n_used = 0;
 
 	char format[GMT_BUFSIZ] = {""}, buffer[GMT_GRID_REMARK_LEN160] = {""};
 
@@ -414,7 +439,7 @@ EXTERN_MSC int GMT_grdgradient (void *V_API, int mode, void *args) {
 	double k_ads = 0.0, diffuse, spec, r_min = DBL_MAX, r_max = -DBL_MAX, scale, sin_Az[2] = {0.0, 0.0};
 	double def_offset = 0.0, def_sigma = 0.0;
 
-	struct GMT_GRID *Surf = NULL, *Slope = NULL, *Out = NULL, *A = NULL;
+	struct GMT_GRID *In = NULL, *Slope = NULL, *Grid = NULL, *A = NULL;
 	struct GRDGRADIENT_CTRL *Ctrl = NULL;
 	struct GMT_CTRL *GMT = NULL, *GMT_cpy = NULL;
 	struct GMT_OPTION *options = NULL;
@@ -442,13 +467,15 @@ EXTERN_MSC int GMT_grdgradient (void *V_API, int mode, void *args) {
 	if (Ctrl->Q.mode & 1) {	/* Read in previous statistics */
 		char sfile[PATH_MAX] = {""};
 		FILE *fp = NULL;
-		GMT_Report (API, GMT_MSG_DEBUG, "Read statistics file [%s] with offset and sigma\n", sfile);
-		if (GMT->session.TMPDIR)
+		if (Ctrl->Q.file)
+			sprintf (sfile, "%s", Ctrl->Q.file);
+		else if (GMT->session.TMPDIR)
 			sprintf (sfile, "%s/grdgradient.stat", GMT->session.TMPDIR);
 		else if (API->tmp_dir)
 			sprintf (sfile, "%s/grdgradient.stat", API->tmp_dir);
 		else
 			sprintf (sfile, "grdgradient.stat");
+		GMT_Report (API, GMT_MSG_DEBUG, "Read statistics file [%s] with offset and sigma\n", sfile);
 		if (access (sfile, F_OK)) {
 			GMT_Report (API, GMT_MSG_ERROR, "Unable to find statistics file from last run [%s]!\n", sfile);
 			Return (GMT_FILE_NOT_FOUND);
@@ -520,55 +547,60 @@ EXTERN_MSC int GMT_grdgradient (void *V_API, int mode, void *args) {
 
 	if (GMT->common.R.active[RSET]) gmt_M_memcpy (wesn, GMT->common.R.wesn, 4, double);	/* Current -R setting, if any */
 
-	if ((Surf = GMT_Read_Data (API, GMT_IS_GRID, GMT_IS_FILE, GMT_IS_SURFACE, GMT_CONTAINER_ONLY, NULL, Ctrl->In.file, NULL)) == NULL) {
+	if ((In = GMT_Read_Data (API, GMT_IS_GRID, GMT_IS_FILE, GMT_IS_SURFACE, GMT_CONTAINER_ONLY, NULL, Ctrl->In.file, NULL)) == NULL) {
 		Return (API->error);
 	}
-	if (gmt_M_is_subset (GMT, Surf->header, wesn)) {	/* Subset requested; make sure wesn matches header spacing */
-		if ((error = gmt_M_err_fail (GMT, gmt_adjust_loose_wesn (GMT, wesn, Surf->header), "")))
+	if (gmt_M_is_subset (GMT, In->header, wesn)) {	/* Subset requested; make sure wesn matches header spacing */
+		if ((error = gmt_M_err_fail (GMT, gmt_adjust_loose_wesn (GMT, wesn, In->header), "")))
 			Return (error);
 	}
-	gmt_grd_init (GMT, Surf->header, options, true);
+	gmt_grd_init (GMT, In->header, options, true);
 
-	if (GMT_Read_Data (API, GMT_IS_GRID, GMT_IS_FILE, GMT_IS_SURFACE, GMT_DATA_ONLY, wesn, Ctrl->In.file, Surf) == NULL) {	/* Get subset */
+	if (GMT_Read_Data (API, GMT_IS_GRID, GMT_IS_FILE, GMT_IS_SURFACE, GMT_DATA_ONLY, wesn, Ctrl->In.file, In) == NULL) {	/* Get subset */
 		Return (API->error);
 	}
 
 	if (Ctrl->A.mode == GRDGRADIENT_VAR) {	/* IGiven 2 grids, make sure they are co-registered and has same size, registration, etc. */
-		if (Surf->header->registration != A->header->registration) {
+		if (In->header->registration != A->header->registration) {
 			GMT_Report (API, GMT_MSG_ERROR, "Input and azimuth grids have different registrations!\n");
 			Return (GMT_RUNTIME_ERROR);
 		}
-		if (!gmt_M_grd_same_shape (GMT, Surf, A)) {
+		if (!gmt_M_grd_same_shape (GMT, In, A)) {
 			GMT_Report (API, GMT_MSG_ERROR, "Input and azimuth grids have different dimensions\n");
 			Return (GMT_RUNTIME_ERROR);
 		}
-		if (!gmt_M_grd_same_region (GMT, Surf, A)) {
+		if (!gmt_M_grd_same_region (GMT, In, A)) {
 			GMT_Report (API, GMT_MSG_ERROR, "Input and azimuth grids have different regions\n");
 			Return (GMT_RUNTIME_ERROR);
 		}
-		if (!gmt_M_grd_same_inc (GMT, Surf, A)) {
+		if (!gmt_M_grd_same_inc (GMT, In, A)) {
 			GMT_Report (API, GMT_MSG_ERROR, "Input and azimuth grids have different intervals\n");
 			Return (GMT_RUNTIME_ERROR);
 		}
 	}
 
-	if (Ctrl->S.active) {	/* Want slope grid */
-		if ((Slope = GMT_Duplicate_Data (API, GMT_IS_GRID, GMT_DUPLICATE_ALLOC, Surf)) == NULL) Return (API->error);
-	}
 #ifdef OPENMP
 #if 0
 	separate = true;	/* Cannot use input grid to hold output grid when doing things in parallel */
 #endif
 #endif
-	new_grid = gmt_set_outgrid (GMT, Ctrl->In.file, separate, 1, Surf, &Out);	/* true if input is a read-only array */
+	new_grid = gmt_set_outgrid (GMT, Ctrl->In.file, separate, 2, In, &Grid);	/* true if input is a read-only array */
+
+	/* If new_grid is true then Grid points to a duplicate of In but will have two boundary rows,columns padding.
+	 * If new_grid is false then Grid simply points to In which presumably has two boundary row,column padding.
+	 * In either case, the algorithm below assumes there is at least one extra row column in Grid */
+
+	if (Ctrl->S.active) {	/* Want slope grid, ensure same padding as Grid */
+		if ((Slope = GMT_Duplicate_Data (API, GMT_IS_GRID, GMT_DUPLICATE_ALLOC, Grid)) == NULL) Return (API->error);
+	}
 
 	if (gmt_M_is_geographic (GMT, GMT_IN) && !Ctrl->E.active) {	/* Flat-Earth approximation */
-		dx_grid = GMT->current.proj.DIST_M_PR_DEG * Surf->header->inc[GMT_X] * cosd ((Surf->header->wesn[YHI] + Surf->header->wesn[YLO]) / 2.0);
-		dy_grid = GMT->current.proj.DIST_M_PR_DEG * Surf->header->inc[GMT_Y];
+		dx_grid = GMT->current.proj.DIST_M_PR_DEG * Grid->header->inc[GMT_X] * cosd ((Grid->header->wesn[YHI] + Grid->header->wesn[YLO]) / 2.0);
+		dy_grid = GMT->current.proj.DIST_M_PR_DEG * Grid->header->inc[GMT_Y];
 	}
 	else {	/* Cartesian */
-		dx_grid = Surf->header->inc[GMT_X];
-		dy_grid = Surf->header->inc[GMT_Y];
+		dx_grid = Grid->header->inc[GMT_X];
+		dy_grid = Grid->header->inc[GMT_Y];
 	}
 	one = (Ctrl->D.active) ? +1.0 : -1.0;	/* With -D we want positive grad direction, not negative as for shading (-A, -E) */
 	x_factor_set = one / (2.0 * dx_grid);
@@ -586,15 +618,15 @@ EXTERN_MSC int GMT_grdgradient (void *V_API, int mode, void *args) {
 		y_factor *= cos (Ctrl->A.azimuth[0]);
 	}
 
-	/* Index offset of 4-star points relative to current node */
-	mx = Surf->header->mx;	/* Need a signed mx for p[3] in line below */
+	/* Index offset of 4-star points relative to current node in Grid */
+	mx = Grid->header->mx;	/* Need a signed mx for p[3] in line below */
 	p[0] = 1;	p[1] = -1;	p[2] = mx;	p[3] = -mx;
 
 	min_gradient = DBL_MAX;	max_gradient = -DBL_MAX;	ave_gradient = 0.0;
 	if (Ctrl->E.mode == 3) {
-		lim_x = Surf->header->wesn[XHI] - Surf->header->wesn[XLO];
-		lim_y = Surf->header->wesn[YHI] - Surf->header->wesn[YLO];
-		lim_z = Surf->header->z_max - Surf->header->z_min;
+		lim_x = Grid->header->wesn[XHI] - Grid->header->wesn[XLO];
+		lim_y = Grid->header->wesn[YHI] - Grid->header->wesn[YLO];
+		lim_z = Grid->header->z_max - Grid->header->z_min;
 		scale = MAX (lim_z, MAX (lim_x, lim_y));
 		lim_x /= scale;	lim_y /= scale;		lim_z /= scale;
 		dx_grid /= lim_x;	dy_grid /= lim_y;
@@ -604,14 +636,14 @@ EXTERN_MSC int GMT_grdgradient (void *V_API, int mode, void *args) {
 #if 0	/* Not active since we have no way to do min/max via OpenMP in C.  So rethink this part */
 #ifdef _OPENMP
 #pragma omp parallel for private(row,ij0,lat,dx_grid,col,ij,n,bad,index,dzdx,dzdy,dzdx2,dzdy2,dzds1,dzds2,output,azim,norm_z,mag,diffuse,spec) \
-	shared(Surf,GMT,Ctrl,x_factor_set,x_factor2_set,sin_Az,new_grid,Out,Slope,y_factor,y_factor2,p0,q0,p0q0_cte,k_ads,s) \
+	shared(In,GMT,Ctrl,x_factor_set,x_factor2_set,sin_Az,new_grid,Grid,Slope,y_factor,y_factor2,p0,q0,p0q0_cte,k_ads,s) \
 	reduction(+:ave_gradient)
 #endif
 #endif
-	for (row = 0, ij0 = 0ULL; row < Surf->header->n_rows; row++) {	/* ij0 is the index in a non-padded grid */
+	for (row = 0, ij0 = 0ULL; row < Grid->header->n_rows; row++) {	/* ij0 is the index in a non-padded grid */
 		if (gmt_M_is_geographic (GMT, GMT_IN) && !Ctrl->E.active) {	/* Evaluate latitude-dependent factors */
-			lat = gmt_M_grd_row_to_y (GMT, row, Surf->header);
-			dx_grid = GMT->current.proj.DIST_M_PR_DEG * Surf->header->inc[GMT_X] * cosd (lat);
+			lat = gmt_M_grd_row_to_y (GMT, row, Grid->header);
+			dx_grid = GMT->current.proj.DIST_M_PR_DEG * Grid->header->inc[GMT_X] * cosd (lat);
 			if (dx_grid > 0.0) x_factor = x_factor_set = one / (2.0 * dx_grid);	/* Use previous value at the poles */
 			if (Ctrl->A.mode == GRDGRADIENT_FIX) {
 				if (Ctrl->A.two) x_factor2 = x_factor * sin_Az[1];
@@ -622,12 +654,11 @@ EXTERN_MSC int GMT_grdgradient (void *V_API, int mode, void *args) {
 			x_factor = x_factor_set;
 			if (Ctrl->A.mode == GRDGRADIENT_FIX && Ctrl->A.two) x_factor2 = x_factor2_set;
 		}
-		for (col = 0; col < Surf->header->n_columns; col++, ij0++) {
-			ij = gmt_M_ijp (Surf->header, row, col);	/* Index into padded grid */
-			for (n = 0, bad = false; !bad && n < 4; n++) if (gmt_M_is_fnan (Surf->data[ij+p[n]])) bad = true;
+		for (col = 0; col < Grid->header->n_columns; col++, ij0++) {
+			ij = gmt_M_ijp (Grid->header, row, col);	/* Index into padded grid (with at least 1 row/col padding) */
+			for (n = 0, bad = false; !bad && n < 4; n++) if (gmt_M_is_fnan (Grid->data[ij+p[n]])) bad = true;
 			if (bad) {	/* One of the 4-star corners = NaN; assign NaN answer and skip to next node */
-				index = (new_grid) ? ij : ij0;
-				Out->data[index] = GMT->session.f_NaN;
+				Grid->data[ij0] = GMT->session.f_NaN;
 				if (Ctrl->S.active) Slope->data[ij] = GMT->session.f_NaN;
 				continue;
 			}
@@ -638,11 +669,11 @@ EXTERN_MSC int GMT_grdgradient (void *V_API, int mode, void *args) {
 			}
 
 			/* We can now evaluate the central finite differences */
-			dzdx = (Surf->data[ij+1] - Surf->data[ij-1]) * x_factor;
-			dzdy = (Surf->data[ij-Surf->header->mx] - Surf->data[ij+Surf->header->mx]) * y_factor;
+			dzdx = (Grid->data[ij+1] - Grid->data[ij-1]) * x_factor;
+			dzdy = (Grid->data[ij-Grid->header->mx] - Grid->data[ij+Grid->header->mx]) * y_factor;
 			if (Ctrl->A.two) {
-				dzdx2 = (Surf->data[ij+1] - Surf->data[ij-1]) * x_factor2;
-				dzdy2 = (Surf->data[ij-Surf->header->mx] - Surf->data[ij+Surf->header->mx]) * y_factor2;
+				dzdx2 = (Grid->data[ij+1] - Grid->data[ij-1]) * x_factor2;
+				dzdy2 = (Grid->data[ij-Grid->header->mx] - Grid->data[ij+Grid->header->mx]) * y_factor2;
 			}
 
 			/* Write output to unused NW corner */
@@ -692,38 +723,37 @@ EXTERN_MSC int GMT_grdgradient (void *V_API, int mode, void *args) {
 				r_min = MIN (r_min, output);
 				r_max = MAX (r_max, output);
 			}
-			index = (new_grid) ? ij : ij0;
-			Out->data[index] = (gmt_grdfloat)output;
+			Grid->data[ij0] = (gmt_grdfloat)output;
 			n_used++;
 		}
 	}
 
-	if (!new_grid)	{	/* We got away with using the input grid by ignoring the pad.  Now we must put the pad back in */
-		gmt_M_memset (Out->header->pad, 4, int);	/* Must set pad to zero first otherwise we cannot add the pad in */
-		Out->header->mx = Out->header->n_columns;	Out->header->my = Out->header->n_rows;	/* Since there is no pad */
-		gmt_grd_pad_on (GMT, Out, GMT->current.io.pad);	/* Now reinstate the pad */
-	}
+	/* Now deal with the fact that the result is unpadded in a padded array */
+	gmt_M_memcpy (orig_pad, Grid->header->pad, 4, unsigned int);	/* This can be either 1/1/1/1/ or 2/2/2/2, depending on circumstances */
+	Grid->header->mx = Grid->header->n_columns;	Grid->header->my = Grid->header->n_rows;	/* Since there is no pad as far as the computed grid is concerned */
+	gmt_M_memset (Grid->header->pad, 4, int);	/* Must set pad to zero first otherwise we cannot add the pad back in */
+	gmt_grd_pad_on (GMT, Grid, orig_pad);	/* Now reinstate the original pad */
 
 	if (gmt_M_is_geographic (GMT, GMT_IN)) {	/* Data is geographic */
 		double sum;
 		/* If the N or S poles are included then we only want a single estimate at these repeating points */
-		if (Out->header->wesn[YLO] == -90.0 && Out->header->registration == GMT_GRID_NODE_REG) {	/* Average all the multiple N pole estimates */
-			for (col = 0, ij = gmt_M_ijp (Out->header, 0, 0), sum = 0.0; col < Out->header->n_columns; col++, ij++) sum += Out->data[ij];
-			sum /= Out->header->n_columns;	/* Average gradient */
-			for (col = 0, ij = gmt_M_ijp (Out->header, 0, 0); col < Out->header->n_columns; col++, ij++) Out->data[ij] = (gmt_grdfloat)sum;
+		if (Grid->header->wesn[YLO] == -90.0 && Grid->header->registration == GMT_GRID_NODE_REG) {	/* Average all the multiple N pole estimates */
+			for (col = 0, ij = gmt_M_ijp (Grid->header, 0, 0), sum = 0.0; col < Grid->header->n_columns; col++, ij++) sum += Grid->data[ij];
+			sum /= Grid->header->n_columns;	/* Average gradient */
+			for (col = 0, ij = gmt_M_ijp (Grid->header, 0, 0); col < Grid->header->n_columns; col++, ij++) Grid->data[ij] = (gmt_grdfloat)sum;
 		}
-		if (Out->header->wesn[YLO] == -90.0 && Out->header->registration == GMT_GRID_NODE_REG) {	/* Average all the multiple S pole estimates */
-			for (col = 0, ij = gmt_M_ijp (Out->header, Out->header->n_rows - 1, 0), sum = 0.0; col < Out->header->n_columns; col++, ij++) sum += Out->data[ij];
-			sum /= Out->header->n_columns;	/* Average gradient */
-			for (col = 0, ij = gmt_M_ijp (Out->header, Out->header->n_rows - 1, 0); col < Out->header->n_columns; col++, ij++) Out->data[ij] = (gmt_grdfloat)sum;
+		if (Grid->header->wesn[YLO] == -90.0 && Grid->header->registration == GMT_GRID_NODE_REG) {	/* Average all the multiple S pole estimates */
+			for (col = 0, ij = gmt_M_ijp (Grid->header, Grid->header->n_rows - 1, 0), sum = 0.0; col < Grid->header->n_columns; col++, ij++) sum += Grid->data[ij];
+			sum /= Grid->header->n_columns;	/* Average gradient */
+			for (col = 0, ij = gmt_M_ijp (Grid->header, Grid->header->n_rows - 1, 0); col < Grid->header->n_columns; col++, ij++) Grid->data[ij] = (gmt_grdfloat)sum;
 		}
 	}
 
 	if (Ctrl->E.active) {	/* data must be scaled to the [-1,1] interval, but we'll do it into [-.95, .95] to not get too bright */
 		scale = 1.0 / (r_max - r_min);
-		gmt_M_grd_loop (GMT, Out, row, col, ij) {
-			if (gmt_M_is_fnan (Out->data[ij])) continue;
-			Out->data[ij] = (gmt_grdfloat)((-1.0 + 2.0 * ((Out->data[ij] - r_min) * scale)) * 0.95);
+		gmt_M_grd_loop (GMT, Grid, row, col, ij) {
+			if (gmt_M_is_fnan (Grid->data[ij])) continue;
+			Grid->data[ij] = (gmt_grdfloat)((-1.0 + 2.0 * ((Grid->data[ij] - r_min) * scale)) * 0.95);
 		}
 	}
 
@@ -740,54 +770,54 @@ EXTERN_MSC int GMT_grdgradient (void *V_API, int mode, void *args) {
 					denom = 1.0 / Ctrl->N.sigma;
 				else {
 					denom = 0.0;
-					gmt_M_grd_loop (GMT, Out, row, col, ij) {
-						if (!gmt_M_is_fnan (Out->data[ij])) denom += pow (Out->data[ij] - ave_gradient, 2.0);
+					gmt_M_grd_loop (GMT, Grid, row, col, ij) {
+						if (!gmt_M_is_fnan (Grid->data[ij])) denom += pow (Grid->data[ij] - ave_gradient, 2.0);
 					}
 					denom = sqrt ((n_used - 1) / denom);
 					Ctrl->N.sigma = 1.0 / denom;
 				}
 				rpi = 2.0 * Ctrl->N.norm / M_PI;
-				gmt_M_grd_loop (GMT, Out, row, col, ij) {
-					if (!gmt_M_is_fnan (Out->data[ij])) Out->data[ij] = (gmt_grdfloat)(rpi * atan ((Out->data[ij] - ave_gradient) * denom) + Ctrl->N.ambient);
+				gmt_M_grd_loop (GMT, Grid, row, col, ij) {
+					if (!gmt_M_is_fnan (Grid->data[ij])) Grid->data[ij] = (gmt_grdfloat)(rpi * atan ((Grid->data[ij] - ave_gradient) * denom) + Ctrl->N.ambient);
 				}
-				Out->header->z_max = rpi * atan ((max_gradient - ave_gradient) * denom) + Ctrl->N.ambient;
-				Out->header->z_min = rpi * atan ((min_gradient - ave_gradient) * denom) + Ctrl->N.ambient;
+				Grid->header->z_max = rpi * atan ((max_gradient - ave_gradient) * denom) + Ctrl->N.ambient;
+				Grid->header->z_min = rpi * atan ((min_gradient - ave_gradient) * denom) + Ctrl->N.ambient;
 			}
 			else if (Ctrl->N.mode == 2) {	/* Exp transformation */
 				if (!Ctrl->N.set[2]) {
 					Ctrl->N.sigma = 0.0;
-					gmt_M_grd_loop (GMT, Out, row, col, ij) {
+					gmt_M_grd_loop (GMT, Grid, row, col, ij) {
 #ifdef DOUBLE_PRECISION_GRID
-						if (!gmt_M_is_fnan (Out->data[ij])) Ctrl->N.sigma += fabs (Out->data[ij]);
+						if (!gmt_M_is_fnan (Grid->data[ij])) Ctrl->N.sigma += fabs (Grid->data[ij]);
 #else
-						if (!gmt_M_is_fnan (Out->data[ij])) Ctrl->N.sigma += fabsf (Out->data[ij]);
+						if (!gmt_M_is_fnan (Grid->data[ij])) Ctrl->N.sigma += fabsf (Grid->data[ij]);
 #endif
 					}
 					Ctrl->N.sigma = M_SQRT2 * Ctrl->N.sigma / n_used;
 				}
 				denom = M_SQRT2 / Ctrl->N.sigma;
-				gmt_M_grd_loop (GMT, Out, row, col, ij) {
-					if (gmt_M_is_fnan (Out->data[ij])) continue;
-					if (Out->data[ij] < ave_gradient) {
-						Out->data[ij] = (gmt_grdfloat)(-Ctrl->N.norm * (1.0 - exp ( (Out->data[ij] - ave_gradient) * denom)) + Ctrl->N.ambient);
+				gmt_M_grd_loop (GMT, Grid, row, col, ij) {
+					if (gmt_M_is_fnan (Grid->data[ij])) continue;
+					if (Grid->data[ij] < ave_gradient) {
+						Grid->data[ij] = (gmt_grdfloat)(-Ctrl->N.norm * (1.0 - exp ( (Grid->data[ij] - ave_gradient) * denom)) + Ctrl->N.ambient);
 					}
 					else {
-						Out->data[ij] = (gmt_grdfloat)( Ctrl->N.norm * (1.0 - exp (-(Out->data[ij] - ave_gradient) * denom)) + Ctrl->N.ambient);
+						Grid->data[ij] = (gmt_grdfloat)( Ctrl->N.norm * (1.0 - exp (-(Grid->data[ij] - ave_gradient) * denom)) + Ctrl->N.ambient);
 					}
 				}
-				Out->header->z_max =  Ctrl->N.norm * (1.0 - exp (-(max_gradient - ave_gradient) * denom)) + Ctrl->N.ambient;
-				Out->header->z_min = -Ctrl->N.norm * (1.0 - exp ( (min_gradient - ave_gradient) * denom)) + Ctrl->N.ambient;
+				Grid->header->z_max =  Ctrl->N.norm * (1.0 - exp (-(max_gradient - ave_gradient) * denom)) + Ctrl->N.ambient;
+				Grid->header->z_min = -Ctrl->N.norm * (1.0 - exp ( (min_gradient - ave_gradient) * denom)) + Ctrl->N.ambient;
 			}
 			else {	/* Linear transformation */
 				if ((max_gradient - ave_gradient) > (ave_gradient - min_gradient))
 					denom = Ctrl->N.norm / (max_gradient - ave_gradient);
 				else
 					denom = Ctrl->N.norm / (ave_gradient - min_gradient);
-				gmt_M_grd_loop (GMT, Out, row, col, ij) {
-					if (!gmt_M_is_fnan (Out->data[ij])) Out->data[ij] = (gmt_grdfloat)((Out->data[ij] - ave_gradient) * denom) + Ctrl->N.ambient;
+				gmt_M_grd_loop (GMT, Grid, row, col, ij) {
+					if (!gmt_M_is_fnan (Grid->data[ij])) Grid->data[ij] = (gmt_grdfloat)((Grid->data[ij] - ave_gradient) * denom) + Ctrl->N.ambient;
 				}
-				Out->header->z_max = (max_gradient - ave_gradient) * denom + Ctrl->N.ambient;
-				Out->header->z_min = (min_gradient - ave_gradient) * denom + Ctrl->N.ambient;
+				Grid->header->z_max = (max_gradient - ave_gradient) * denom + Ctrl->N.ambient;
+				Grid->header->z_min = (min_gradient - ave_gradient) * denom + Ctrl->N.ambient;
 			}
 		}
 	}
@@ -815,9 +845,9 @@ EXTERN_MSC int GMT_grdgradient (void *V_API, int mode, void *args) {
 				strcpy (buffer, "Directions of grad (z) [uphill direction]");
 		}
 
-		if (GMT_Set_Comment (API, GMT_IS_GRID, GMT_COMMENT_IS_OPTION | GMT_COMMENT_IS_COMMAND, options, Out)) Return (API->error);
-		if (GMT_Set_Comment (API, GMT_IS_GRID, GMT_COMMENT_IS_REMARK, buffer, Out)) Return (API->error);
-		if (Ctrl->G.active && GMT_Write_Data (API, GMT_IS_GRID, GMT_IS_FILE, GMT_IS_SURFACE, GMT_CONTAINER_AND_DATA, NULL, Ctrl->G.file, Out) != GMT_NOERROR) {
+		if (GMT_Set_Comment (API, GMT_IS_GRID, GMT_COMMENT_IS_OPTION | GMT_COMMENT_IS_COMMAND, options, Grid)) Return (API->error);
+		if (GMT_Set_Comment (API, GMT_IS_GRID, GMT_COMMENT_IS_REMARK, buffer, Grid)) Return (API->error);
+		if (Ctrl->G.active && GMT_Write_Data (API, GMT_IS_GRID, GMT_IS_FILE, GMT_IS_SURFACE, GMT_CONTAINER_AND_DATA, NULL, Ctrl->G.file, Grid) != GMT_NOERROR) {
 			Return (API->error);
 		}
 
@@ -833,13 +863,15 @@ EXTERN_MSC int GMT_grdgradient (void *V_API, int mode, void *args) {
 	if (Ctrl->Q.mode == 2) {	/* Write the new statistics */
 		char sfile[PATH_MAX] = {""};
 		FILE *fp = NULL;
-		GMT_Report (API, GMT_MSG_DEBUG, "Create statistics file [%s] with offset and sigma\n", sfile);
-		if (GMT->session.TMPDIR)
+		if (Ctrl->Q.file)
+			sprintf (sfile, "%s", Ctrl->Q.file);
+		else if (GMT->session.TMPDIR)
 			sprintf (sfile, "%s/grdgradient.stat", GMT->session.TMPDIR);
 		else if (API->tmp_dir)
 			sprintf (sfile, "%s/grdgradient.stat", API->tmp_dir);
 		else
 			sprintf (sfile, "grdgradient.stat");
+		GMT_Report (API, GMT_MSG_DEBUG, "Create statistics file [%s] with offset and sigma\n", sfile);
 		if ((fp = fopen (sfile, "w")) == NULL) {
 			GMT_Report (API, GMT_MSG_ERROR, "Cannot create statistics file from this run [%s]!\n", sfile);
 			Return (GMT_ERROR_ON_FOPEN);
