@@ -1,6 +1,6 @@
 /*--------------------------------------------------------------------
  *
- *	Copyright (c) 1991-2020 by the GMT Team (https://www.generic-mapping-tools.org/team.html)
+ *	Copyright (c) 1991-2021 by the GMT Team (https://www.generic-mapping-tools.org/team.html)
  *	See LICENSE.TXT file for copying and redistribution conditions.
  *
  *	This program is free software; you can redistribute it and/or modify
@@ -85,13 +85,13 @@ static int usage (struct GMTAPI_CTRL *API, int level) {
 	const char *name = gmt_show_name_and_purpose (API, THIS_MODULE_LIB, THIS_MODULE_CLASSIC_NAME, THIS_MODULE_PURPOSE);
 	if (level == GMT_MODULE_PURPOSE) return (GMT_NOERROR);
 	GMT_Message (API, GMT_TIME_NONE, "usage: %s begin -D%s |\n\t-D%s\n", name, GMT_INSET_A, GMT_INSET_B);
-	GMT_Message (API, GMT_TIME_NONE, "\t[-F%s] [-M<margins>] [-N] [%s] [%s]\n\n", GMT_PANEL, GMT_V_OPT, GMT_PAR_OPT);
+	GMT_Message (API, GMT_TIME_NONE, "\t[-F%s]\n\t[-M<margins>] [-N] [%s] [%s]\n\n", GMT_PANEL, GMT_V_OPT, GMT_PAR_OPT);
 	GMT_Message (API, GMT_TIME_NONE, "usage: %s end [%s]\n\n", name, GMT_V_OPT);
 
 	if (level == GMT_SYNOPSIS) return (GMT_MODULE_SYNOPSIS);
 
 	gmt_mapinset_syntax (API->GMT, 'D', "Design a simple map inset as specified below:");
-	GMT_Message (API, GMT_TIME_NONE, "\n\tOPTIONS:\n");
+	GMT_Message (API, GMT_TIME_NONE, "\n  OPTIONAL ARGUMENTS:\n");
 	gmt_mappanel_syntax (API->GMT, 'F', "Specify a rectangular panel behind the map inset.", 3);
 	GMT_Message (API, GMT_TIME_NONE, "\t-M Allows for space around the inset. Append a uniform <margin>, separate <xmargin>/<ymargin>,\n");
 	GMT_Message (API, GMT_TIME_NONE, "\t   or individual <wmargin>/<emargin>/<smargin>/<nmargin> for each side [no margin].\n");
@@ -111,10 +111,11 @@ static int parse (struct GMT_CTRL *GMT, struct INSET_CTRL *Ctrl, struct GMT_OPTI
 
 	unsigned int n_errors = 0, k;
 	struct GMT_OPTION *opt = NULL;
+	struct GMTAPI_CTRL *API = GMT->parent;
 
 	opt = options;	/* The first argument is the inset command */
 	if (opt->option != GMT_OPT_INFILE) {
-		GMT_Report (GMT->parent, GMT_MSG_ERROR, "No inset command given\n");
+		GMT_Report (API, GMT_MSG_ERROR, "No inset command given\n");
 		return GMT_PARSE_ERROR;
 	}
 	if (!strncmp (opt->arg, "begin", 5U)) {	/* Initiate new inset */
@@ -123,7 +124,7 @@ static int parse (struct GMT_CTRL *GMT, struct INSET_CTRL *Ctrl, struct GMT_OPTI
 	else if (!strncmp (opt->arg, "end", 3U))
 		Ctrl->In.mode = INSET_END;
 	else {
-		GMT_Report (GMT->parent, GMT_MSG_ERROR, "Not an inset command: %s\n", opt->arg);
+		GMT_Report (API, GMT_MSG_ERROR, "Not an inset command: %s\n", opt->arg);
 		return GMT_PARSE_ERROR;
 	}
 	opt = opt->next;	/* Position to the next argument */
@@ -133,11 +134,13 @@ static int parse (struct GMT_CTRL *GMT, struct INSET_CTRL *Ctrl, struct GMT_OPTI
 		switch (opt->option) {
 
 			case 'D':	/* Draw map inset */
+				n_errors += gmt_M_repeated_module_option (API, Ctrl->D.active);
 				Ctrl->D.active = true;
 				n_errors += gmt_getinset (GMT, 'D', opt->arg, &Ctrl->D.inset);
 				break;
 
 			case 'F':
+				n_errors += gmt_M_repeated_module_option (API, Ctrl->F.active);
 				Ctrl->F.active = true;
 				if (gmt_getpanel (GMT, opt->option, opt->arg, &(Ctrl->D.inset.panel))) {
 					gmt_mappanel_syntax (GMT, 'F', "Specify a rectangular panel for the map inset", 3);
@@ -146,13 +149,14 @@ static int parse (struct GMT_CTRL *GMT, struct INSET_CTRL *Ctrl, struct GMT_OPTI
 				break;
 
 			case 'M':	/* inset margins */
+				n_errors += gmt_M_repeated_module_option (API, Ctrl->M.active);
 				Ctrl->M.active = true;
 				if (opt->arg[0] == 0) {	/* Gave nothing */
-					GMT_Report (GMT->parent, GMT_MSG_ERROR, "Option -M: No margins given.\n");
+					GMT_Report (API, GMT_MSG_ERROR, "Option -M: No margins given.\n");
 					n_errors++;
 				}
 				else {	/* Process 1, 2, or 4 margin values */
-					k = GMT_Get_Values (GMT->parent, opt->arg, Ctrl->M.margin, 4);
+					k = GMT_Get_Values (API, opt->arg, Ctrl->M.margin, 4);
 					if (k == 1)	/* Same page margin in all directions */
 						Ctrl->M.margin[XHI] = Ctrl->M.margin[YLO] = Ctrl->M.margin[YHI] = Ctrl->M.margin[XLO];
 					else if (k == 2) {	/* Separate page margins in x and y */
@@ -160,7 +164,7 @@ static int parse (struct GMT_CTRL *GMT, struct INSET_CTRL *Ctrl, struct GMT_OPTI
 						Ctrl->M.margin[XHI] = Ctrl->M.margin[XLO];
 					}
 					else if (k != 4) {
-						GMT_Report (GMT->parent, GMT_MSG_ERROR, "Option -M: Bad number of margins given.\n");
+						GMT_Report (API, GMT_MSG_ERROR, "Option -M: Bad number of margins given.\n");
 						n_errors++;
 					}
 					/* Since GMT_Get_Values returns in default project length unit, convert to inch */
@@ -169,6 +173,7 @@ static int parse (struct GMT_CTRL *GMT, struct INSET_CTRL *Ctrl, struct GMT_OPTI
 				break;
 
 			case 'N':	/* Turn off clipping  */
+				n_errors += gmt_M_repeated_module_option (API, Ctrl->N.active);
 				Ctrl->N.active = true;
 				break;
 
@@ -180,27 +185,25 @@ static int parse (struct GMT_CTRL *GMT, struct INSET_CTRL *Ctrl, struct GMT_OPTI
 	}
 
 	if (Ctrl->In.mode == INSET_BEGIN) {
-		/* Was -R -J given */
+		/* Was -R -J given? */
+		n_errors += gmt_M_check_condition (GMT, !Ctrl->D.active, "Option -D is required for gmt inset begin\n");
 		n_errors += gmt_M_check_condition (GMT, GMT->common.J.active && !GMT->common.R.active[RSET], "Option -J: Requires -R as well!\n");
-		if (GMT->common.J.active) {	/* Compute map height */
-			if (gmt_M_err_pass (GMT, gmt_map_setup (GMT, GMT->common.R.wesn), "")) n_errors++;
-		}
 	}
 	else {	/* gmt inset end was given, when -D -F -M -N are not allowed */
 		if (Ctrl->D.active) {
-			GMT_Report (GMT->parent, GMT_MSG_ERROR, "Option -D: Not valid for gmt inset end.\n");
+			GMT_Report (API, GMT_MSG_ERROR, "Option -D: Not valid for gmt inset end.\n");
 			n_errors++;
 		}
 		if (Ctrl->F.active) {
-			GMT_Report (GMT->parent, GMT_MSG_ERROR, "Option -F: Not valid for gmt inset end.\n");
+			GMT_Report (API, GMT_MSG_ERROR, "Option -F: Not valid for gmt inset end.\n");
 			n_errors++;
 		}
 		if (Ctrl->M.active) {
-			GMT_Report (GMT->parent, GMT_MSG_ERROR, "Option -M: Not valid for gmt inset end.\n");
+			GMT_Report (API, GMT_MSG_ERROR, "Option -M: Not valid for gmt inset end.\n");
 			n_errors++;
 		}
 		if (Ctrl->N.active) {
-			GMT_Report (GMT->parent, GMT_MSG_ERROR, "Option -N: Not valid for gmt inset end.\n");
+			GMT_Report (API, GMT_MSG_ERROR, "Option -N: Not valid for gmt inset end.\n");
 			n_errors++;
 		}
 	}
@@ -213,12 +216,14 @@ static int parse (struct GMT_CTRL *GMT, struct INSET_CTRL *Ctrl, struct GMT_OPTI
 
 EXTERN_MSC int GMT_inset (void *V_API, int mode, void *args) {
 	int error = 0, fig, k;
-	bool exist;
-	char file[PATH_MAX] = {""}, ffile[PATH_MAX] = {""}, Bopts[GMT_LEN256] = {""};
+	bool exist, got_RJ = false;
+	char tag[GMT_LEN16] = {""}, file[PATH_MAX] = {""}, ffile[PATH_MAX] = {""}, Bopts[GMT_LEN256] = {""};
+	char inset_history[GMT_LEN256] = {""};
+	double dim[2] = {0.0, 0.0};
 	FILE *fp = NULL;
 	struct GMT_CTRL *GMT = NULL, *GMT_cpy = NULL;
 	struct PSL_CTRL *PSL = NULL;		/* General PSL internal parameters */
-	struct GMT_OPTION *options = NULL;
+	struct GMT_OPTION *options = NULL, *R = NULL, *J = NULL;
 	struct INSET_CTRL *Ctrl = NULL;
 	struct GMTAPI_CTRL *API = gmt_get_api_ptr (V_API);	/* Cast from void to GMTAPI_CTRL pointer */
 
@@ -234,6 +239,41 @@ EXTERN_MSC int GMT_inset (void *V_API, int mode, void *args) {
 		bailout (GMT_NOT_MODERN_MODE);
 	}
 
+	/* Check if -R -J was given explicitly on the command line to define the inset size */
+	if ((R = GMT_Find_Option (API, 'R', options)) && (J = GMT_Find_Option (API, 'J', options))) {	/* Yes they were */
+		/* Create a mapproject command with -R -J -W -Di to get the dimensions of the inset */
+		char ofile[GMT_VF_LEN] = {""}, cmd[GMT_LEN128] = {""};
+		struct GMT_DATASET *Out = NULL;
+
+		/* We also will need to make a special gmt.history file for commands to follow in the inset */
+		sprintf(inset_history, "# GMT 6 Session common arguments shelf\nBEGIN GMT %s\nJ\t%c\nJ%c\t%s\nR\t%s\n", GMT_version(), J->arg[0], J->arg[0], J->arg, R->arg);
+		/* Open Virtual file to hold the result from mapproject */
+		if (GMT_Open_VirtualFile (API, GMT_IS_DATASET, GMT_IS_POINT, GMT_OUT|GMT_IS_REFERENCE, NULL, ofile) == GMT_NOTSET)
+			bailout (API->error);
+		/* Build the mapproject command that takes no input but returns dimensions in inches. Turn off updating of history file */
+		sprintf (cmd, "-R%s -J%s -W -Di ->%s --GMT_HISTORY=false", R->arg, J->arg, ofile);
+		if (GMT_Call_Module (API, "mapproject", GMT_MODULE_CMD, cmd) != GMT_OK)	/* Get the inset size via mapproject */
+			return (API->error);
+		/* Retrieve the answers */
+		if ((Out = GMT_Read_VirtualFile (API, ofile)) == NULL)
+			bailout (API->error);
+		/* Store the size in dim */
+		dim[GMT_X] = Out->table[0]->segment[0]->data[GMT_X][0];
+		dim[GMT_Y] = Out->table[0]->segment[0]->data[GMT_Y][0];
+		/* Close virtual file and free the Out dataset */
+		if (GMT_Close_VirtualFile (API, ofile) != GMT_NOERROR)
+			bailout (API->error);
+		if (GMT_Destroy_Data (API, &Out) != GMT_OK)
+			bailout (API->error);
+		/* Remove the -R -J options from the options list to process below (since we expect -R -J to come from history */
+		if (GMT_Delete_Option (API, R, &options))
+			bailout (API->error);
+		if (GMT_Delete_Option (API, J, &options))
+			bailout (API->error);
+		gmt_reload_history (API->GMT);	/* Prevent gmt from copying previous -R -J history to this inset */
+		got_RJ = true;	/* Meaning we got the -R -J to be used inside the inset before the inset was finalized */
+	}
+
 	/* Parse the command-line arguments */
 
 	if ((GMT = gmt_init_module (API, THIS_MODULE_LIB, THIS_MODULE_CLASSIC_NAME, THIS_MODULE_KEYS, THIS_MODULE_NEEDS, NULL, &options, &GMT_cpy)) == NULL) bailout (API->error); /* Save current state */
@@ -243,6 +283,14 @@ EXTERN_MSC int GMT_inset (void *V_API, int mode, void *args) {
 
 	/*---------------------------- This is the inset main code ----------------------------*/
 
+	if (got_RJ) {	/* Copy the saved inset size determined from -R -J above */
+		if (Ctrl->D.inset.dim[GMT_X] > 0.0 || Ctrl->D.inset.dim[GMT_Y] > 0.0) {
+			GMT_Report (API, GMT_MSG_ERROR, "Cannot define inset size both via -D..,+w as well as -R -J\n");
+			Return (GMT_RUNTIME_ERROR);
+		}
+		gmt_M_memcpy (Ctrl->D.inset.dim, dim, 2U, double);
+	}
+
 	fig = gmt_get_current_figure (API);	/* Get current figure number */
 
 	if ((k = gmt_set_psfilename (GMT)) == GMT_NOTSET) {	/* Get hidden file name for PS */
@@ -252,7 +300,7 @@ EXTERN_MSC int GMT_inset (void *V_API, int mode, void *args) {
 
 	sprintf (file, "%s/gmt.inset.%d", API->gwf_dir, fig);	/* Inset information file */
 
-	exist = !access (file, F_OK);	/* Determine if inset information file exists */
+	exist = !access (file, F_OK);	/* Determine if an inset information file exists */
 	if (Ctrl->In.mode == INSET_BEGIN && exist) {	/* Inset information file already exists which is a failure */
 		GMT_Report (API, GMT_MSG_ERROR, "In begin mode but inset information file already exists: %s\n", file);
 		Return (GMT_RUNTIME_ERROR);
@@ -265,12 +313,17 @@ EXTERN_MSC int GMT_inset (void *V_API, int mode, void *args) {
 	if ((PSL = gmt_plotinit (GMT, options)) == NULL) Return (GMT_RUNTIME_ERROR);
 
 	if (Ctrl->In.mode == INSET_BEGIN) {	/* Determine and save inset attributes */
-		/* Here we need to compute dimensions and save those plus current -R -J to the inset information file,
-		 * then inset a gsave command, translate origin to the inset, adjust for any margins, compute new scales/widths and maybe
-		 * draw the panel. */
+		/* Here we need to compute dimensions and save those plus current plot -R -J to the inset information file,
+		 * then inset a gsave command, translate origin to the inset, adjust for any margins, compute new scales/widths
+		 * and maybe draw the panel. */
 
 		char *cmd = NULL;
 
+		/* Was -R -J given via history? */
+		if (GMT->common.J.active && gmt_map_setup (GMT, GMT->common.R.wesn)) {	/* Compute map height */
+			Return (GMT_RUNTIME_ERROR);
+		}
+
 		/* OK, no other inset set for this figure (or panel).  Save graphics state before we draw the inset */
 		PSL_command (PSL, "V %% Begin inset\n");
 
@@ -281,7 +334,7 @@ EXTERN_MSC int GMT_inset (void *V_API, int mode, void *args) {
 		PSL_setorigin (PSL, Ctrl->D.inset.refpoint->x + Ctrl->M.margin[XLO], Ctrl->D.inset.refpoint->y + Ctrl->M.margin[YLO], 0.0, PSL_FWD);	/* Shift plot a bit */
 
 		/* First get the -B options in place before inset was called */
-		sprintf (ffile, "%s/gmt.frame", API->gwf_dir);
+		sprintf (ffile, "%s/gmt.frame.%d", API->gwf_dir, fig);
 		if ((fp = fopen (ffile, "r")) == NULL)
 			GMT_Report (API, GMT_MSG_INFORMATION, "No file %s with frame information - no adjustments made\n", ffile);
 		else {
@@ -309,12 +362,23 @@ EXTERN_MSC int GMT_inset (void *V_API, int mode, void *args) {
 		fclose (fp);
 		GMT_Report (API, GMT_MSG_DEBUG, "inset: Wrote inset settings to information file %s\n", file);
 		gmt_reset_history (GMT);	/* Prevent gmt from copying previous -R -J history to this inset */
+
+		if (got_RJ) {	/* Must write the given -R -J to inset history so subsequent commands can use them */
+			gmt_hierarchy_tag (API, GMT_HISTORY_FILE, GMT_OUT, tag);
+			sprintf (ffile, "%s/%s%s", API->gwf_dir, GMT_HISTORY_FILE, tag);
+			if ((fp = fopen (ffile, "w")) == NULL) {	/* Not good */
+				GMT_Report (API, GMT_MSG_ERROR, "Cannot create inset history file %s\n", ffile);
+				Return (GMT_ERROR_ON_FOPEN);
+			}
+			fprintf (fp, "%s", inset_history);	/* Write just the -R -J history */
+			fclose (fp);
+		}
 	}
 	else {	/* INSET_END */
 		/* Here we need to finish the inset with a grestore and restate the original -R -J in the history file,
 		 * and finally remove the inset information file */
 
-		char tag[GMT_LEN16] = {""}, legend_justification[4] = {""}, pen[GMT_LEN32] = {""}, fill[GMT_LEN32] = {""}, off[GMT_LEN32] = {""};
+		char legend_justification[4] = {""}, pen[GMT_LEN32] = {""}, fill[GMT_LEN32] = {""}, off[GMT_LEN32] = {""};
 		double legend_width = 0.0, legend_scale = 1.0;
 
 		if (gmt_get_legend_info (API, &legend_width, &legend_scale, legend_justification, pen, fill, off)) {	/* Unplaced legend file */
@@ -339,18 +403,18 @@ EXTERN_MSC int GMT_inset (void *V_API, int mode, void *args) {
 		sprintf (ffile, "%s/%s.%s", API->gwf_dir, GMT_HISTORY_FILE, tag);
 		gmt_remove_file (GMT, ffile);
 		/* Restore the old frame B setting to what it was before inset begin was called, if any */
-		if ((fp = fopen (file, "r"))) {	/* There is a gmt.frame file */
+		if ((fp = fopen (file, "r"))) {	/* There is a gmt.frame.<fig> file */
 			while (fgets (Bopts, GMT_LEN256, fp) && strncmp (Bopts, "# FRAME: ", 9U));	/* Wind to reading the frame setting */
 			gmt_chop (Bopts);
-			fclose (fp);	/* Done reading the gmt.frame file */
+			fclose (fp);	/* Done reading the gmt.frame.<fig> file */
 			if (!strncmp (Bopts, "# FRAME: ", 9U) && strlen (Bopts) > 9 && Bopts[9]) {	/* Got a previously saved -B frame setting */
-				sprintf (ffile, "%s/gmt.frame", API->gwf_dir);
+				sprintf (ffile, "%s/gmt.frame.%d", API->gwf_dir, fig);
 				if ((fp = fopen (ffile, "w")) == NULL) {	/* Not good */
 					GMT_Report (API, GMT_MSG_ERROR, "Cannot create frame file %s\n", ffile);
 					Return (GMT_ERROR_ON_FOPEN);
 				}
 				GMT_Report (API, GMT_MSG_DEBUG, "inset: Restore previous frame in %s\n", ffile);
-				/* Restore the previous frame setting in gmt.frame */
+				/* Restore the previous frame setting in gmt.frame.<fig> */
 				fprintf (fp, "%s\n", &Bopts[9]);
 				fclose (fp);
 			}
