@@ -1,6 +1,6 @@
 /*--------------------------------------------------------------------
  *
- *	Copyright (c) 1991-2019 by the GMT Team (https://www.generic-mapping-tools.org/team.html)
+ *	Copyright (c) 1991-2021 by the GMT Team (https://www.generic-mapping-tools.org/team.html)
  *	See LICENSE.TXT file for copying and redistribution conditions.
  *
  *	This program is free software; you can redistribute it and/or modify
@@ -17,7 +17,7 @@
 
 /*!
  * \file okbfuns.h
- * \brief  
+ * \brief
  */
 
 #ifndef OKBFUNS_H
@@ -40,13 +40,13 @@ struct LOC_OR {
 
 struct MAG_PARAM {
 	double	rim[3];
-} *okabe_mag_param;
+};
 
 struct MAG_VAR {		/* Used when only the modulus of magnetization varies */
 	double	rk[3];
-} *okabe_mag_var;
+};
 
 EXTERN_MSC double okabe (struct GMT_CTRL *GMT, double x_o, double y_o, double z_o, double rho, bool is_grav,
-		struct BODY_DESC bd_desc, struct BODY_VERTS *bd_vert, unsigned int km, unsigned int pm, struct LOC_OR *loc_or);
+		struct BODY_DESC bd_desc, struct BODY_VERTS *bd_vert, unsigned int km, unsigned int pm, struct LOC_OR *loc_or, struct MAG_PARAM *okabe_mag_param, struct MAG_VAR *okabe_mag_var);
 
 #endif /* OKBFUNS_H */
