@@ -1,6 +1,6 @@
 /*--------------------------------------------------------------------
  *
- *	Copyright (c) 1991-2019 by the GMT Team (https://www.generic-mapping-tools.org/team.html)
+ *	Copyright (c) 1991-2021 by the GMT Team (https://www.generic-mapping-tools.org/team.html)
  *	See LICENSE.TXT file for copying and redistribution conditions.
  *
  *	This program is free software; you can redistribute it and/or modify
@@ -17,7 +17,7 @@
 
 /*!
  * \file talwani.h
- * \brief  
+ * \brief
  */
 
 #ifndef TALWANI_H
@@ -32,10 +32,10 @@
 
 #include "../mgd77/mgd77_IGF_coeffs.h"	/* Normal gravity coefficients */
 
-GMT_LOCAL double g_normal (double lat) {
+static double g_normal (double lat) {
 	/* IAG 1980 model */
 	double slat2 = sind (lat);
-	slat2 *= slat2;	
+	slat2 *= slat2;
 	return (MGD77_IGF80_G0 * ((1.0 + MGD77_IGF80_G1 * slat2) / sqrt (1.0 - MGD77_IGF80_G2 * slat2)) / SI_TO_MGAL);
 }
 
