@@ -1,6 +1,6 @@
 /*--------------------------------------------------------------------
  *
- *	Copyright (c) 1991-2020 by the GMT Team (https://www.generic-mapping-tools.org/team.html)
+ *	Copyright (c) 1991-2021 by the GMT Team (https://www.generic-mapping-tools.org/team.html)
  *	See LICENSE.TXT file for copying and redistribution conditions.
  *
  *	This program is free software; you can redistribute it and/or modify
@@ -35,7 +35,7 @@
 #define THIS_MODULE_PURPOSE	"Resample 1-D table data using splines"
 #define THIS_MODULE_KEYS	"<D{,ND(,TD(,>D}"
 #define THIS_MODULE_NEEDS	""
-#define THIS_MODULE_OPTIONS "->Vbdefghioqs" GMT_OPT("HMm")
+#define THIS_MODULE_OPTIONS "->Vbdefghijoqsw" GMT_OPT("HMm")
 
 #define INT_1D_CART	0	/* Regular 1-D interpolation */
 #define INT_2D_CART	1	/* Cartesian 2-D path interpolation */
@@ -46,8 +46,8 @@ struct SAMPLE1D_CTRL {
 		bool active;
 		char *file;
 	} Out;
-	struct SAMPLE1D_A {	/* -A[f|m|p|r|R|l][+l] */
-		bool active, loxo;
+	struct SAMPLE1D_A {	/* -A[f|m|p|r|R|l][+d][+l] */
+		bool active, loxo, delete;
 		enum GMT_enum_track mode;
 	} A;
 	struct SAMPLE1D_F {	/* -Fl|a|c|n|s<p>[+d1|2] */
@@ -98,42 +98,55 @@ static int usage (struct GMTAPI_CTRL *API, int level) {
 
 	const char *name = gmt_show_name_and_purpose (API, THIS_MODULE_LIB, THIS_MODULE_CLASSIC_NAME, THIS_MODULE_PURPOSE);
 	if (level == GMT_MODULE_PURPOSE) return (GMT_NOERROR);
-	GMT_Message (API, GMT_TIME_NONE, "usage: %s [<table>] [-A[f|m|p|r|R]+l] [-Fl|a|c|n|s<p>][+d1|2] [-N<time_col>]\n", name);
-	GMT_Message (API, GMT_TIME_NONE, "\t-T[<min>/<max>/]<inc>[+n|a] [%s] [-W<w_col>] [%s] [%s]\n\t[%s] [%s] [%s]\n\t[%s] [%s]\n\t[%s] [%s] [%s] [%s] [%s]\n\n",
-	             GMT_V_OPT, GMT_b_OPT, GMT_d_OPT, GMT_e_OPT, GMT_f_OPT, GMT_g_OPT, GMT_h_OPT, GMT_i_OPT, GMT_j_OPT, GMT_o_OPT, GMT_q_OPT, GMT_s_OPT, GMT_PAR_OPT);
+	GMT_Usage (API, 0, "usage: %s [<table>] [-A[f|m|p|r|R][+d][+l]] [-Fl|a|c|n|s<p>[+d1|2]] [-N<time_col>] "
+		"[-T[<min>/<max>/]<inc>[+i|n][+a][+t][+u]] [%s] [-W<w_col>] [%s] [%s] [%s] [%s] [%s] [%s] [%s] [%s] [%s] [%s] [%s] [%s] [%s]\n",
+		name, GMT_V_OPT, GMT_b_OPT, GMT_d_OPT, GMT_e_OPT, GMT_f_OPT, GMT_g_OPT, GMT_h_OPT, GMT_i_OPT, GMT_j_OPT,
+		GMT_o_OPT, GMT_q_OPT, GMT_s_OPT, GMT_w_OPT, GMT_PAR_OPT);
 
 	if (level == GMT_SYNOPSIS) return (GMT_MODULE_SYNOPSIS);
 
-	GMT_Message (API, GMT_TIME_NONE, "\tOPTIONS:\n");
+	GMT_Usage (API, -1, "Note: The independent variable column (see -N) must be monotonically in/de-creasing.");
+	GMT_Message (API, GMT_TIME_NONE, "\n  REQUIRED ARGUMENTS:\n");
 	GMT_Option (API, "<");
-	GMT_Message (API, GMT_TIME_NONE, "\t   The independent variable (see -N) must be monotonically in/de-creasing.\n");
-	GMT_Message (API, GMT_TIME_NONE, "\t-A Controls how the input track in <table> is resampled when increment has a unit appended:\n");
-	GMT_Message (API, GMT_TIME_NONE, "\t   f: Keep original points, but add intermediate points if needed [Default].\n");
-	GMT_Message (API, GMT_TIME_NONE, "\t   m: Same, but first follow meridian (along y) then parallel (along x).\n");
-	GMT_Message (API, GMT_TIME_NONE, "\t   p: Same, but first follow parallel (along x) then meridian (along y).\n");
-	GMT_Message (API, GMT_TIME_NONE, "\t   r: Resample at equidistant locations; input points not necessarily included.\n");
-	GMT_Message (API, GMT_TIME_NONE, "\t   R: Same, but adjust given spacing to fit the track length exactly.\n");
-	GMT_Message (API, GMT_TIME_NONE, "\t   Append +l to compute distances along rhumblines (loxodromes) [no].\n");
-	GMT_Message (API, GMT_TIME_NONE, "\t-F Set the interpolation mode.  Choose from:\n");
-	GMT_Message (API, GMT_TIME_NONE, "\t   l Linear interpolation.\n");
-	GMT_Message (API, GMT_TIME_NONE, "\t   a Akima spline interpolation.\n");
-	GMT_Message (API, GMT_TIME_NONE, "\t   c Cubic spline interpolation.\n");
-	GMT_Message (API, GMT_TIME_NONE, "\t   n No interpolation (nearest point).\n");
-	GMT_Message (API, GMT_TIME_NONE, "\t   s Smooth spline interpolation (append fit parameter p).\n");
-	GMT_Message (API, GMT_TIME_NONE, "\t   Optionally, append +d1 for 1st derivative or +d2 for 2nd derivative.\n");
-	GMT_Message (API, GMT_TIME_NONE, "\t   [Default is -F%c].\n", type[API->GMT->current.setting.interpolant]);
-	GMT_Message (API, GMT_TIME_NONE, "\t-N Give column number of the independent variable (time) [Default is 0 (first)].\n");
-	GMT_Message (API, GMT_TIME_NONE, "\t-T Make evenly spaced output time steps from <min> to <max> by <inc>.\n");
-	GMT_Message (API, GMT_TIME_NONE, "\t   Append +n to indicate <inc> is the number of t-values to produce over the range instead.\n");
-	GMT_Message (API, GMT_TIME_NONE, "\t   For absolute time resampling, append a valid time unit (%s) to the increment and add +t.\n", GMT_TIME_UNITS_DISPLAY);
-	GMT_Message (API, GMT_TIME_NONE, "\t   For spatial resampling with distance computed from the first two columns, specify increment as\n");
-	GMT_Message (API, GMT_TIME_NONE, "\t   <inc> and append a geospatial distance unit (%s) or c (for Cartesian distances).\n", GMT_LEN_UNITS_DISPLAY);
-	GMT_Message (API, GMT_TIME_NONE, "\t   See -A to control how the spatial resampling is done.\n");
-	GMT_Message (API, GMT_TIME_NONE, "\t   Optionally, append +a to add such internal distances as a final output column [no distances added].\n");
-	GMT_Message (API, GMT_TIME_NONE, "\t   Alternatively, give a file with output times in the first column, or a comma-separated list.\n");
+	GMT_Message (API, GMT_TIME_NONE, "  OPTIONAL ARGUMENTS:\n");
+	GMT_Usage (API, 1, "\n-A[f|m|p|r|R][+d][+l]");
+	GMT_Usage (API, -2, "Control how input track in <table> is resampled when increment has a unit appended:");
+	GMT_Usage (API, 3, "f: Keep original points, but add intermediate points if needed [Default].");
+	GMT_Usage (API, 3, "m: Same, but first follow meridian (along y) then parallel (along x).");
+	GMT_Usage (API, 3, "p: Same, but first follow parallel (along x) then meridian (along y).");
+	GMT_Usage (API, 3, "r: Resample at equidistant locations; input points not necessarily included.");
+	GMT_Usage (API, 3, "R: Same, but adjust given spacing to fit the track length exactly.");
+	GMT_Usage (API, -2, "Optional modifiers:");
+	GMT_Usage (API, 3, "+d Skip records that has no increase in <time_col> value [no skipping].");
+	GMT_Usage (API, 3, "+l Compute distances along rhumblines (loxodromes) [no].");
+	GMT_Usage (API, -2, "Note: +l uses spherical calculations - cannot be combined with -je.");
+	GMT_Usage (API, 1, "\n-Fl|a|c|n|s<p>[+d1|2]");
+	GMT_Usage (API, -2, "Set the interpolation mode.  Choose from:");
+	GMT_Usage (API, 3, "l: Linear interpolation.");
+	GMT_Usage (API, 3, "a: Akima spline interpolation.");
+	GMT_Usage (API, 3, "c: Cubic spline interpolation.");
+	GMT_Usage (API, 3, "n: No interpolation (nearest point).");
+	GMT_Usage (API, 3, "s: Smooth spline interpolation (append fit parameter <p>).");
+	GMT_Usage (API, -2, "Optionally, request a spline derivative via a modifier:");
+	GMT_Usage (API, 3, "+d Append 1 for 1st derivative or 2 for 2nd derivative.");
+	GMT_Usage (API, -2, "[Default is -F%c].", type[API->GMT->current.setting.interpolant]);
+	GMT_Usage (API, 1, "\n-N<time_col>");
+	GMT_Usage (API, -2, "Give column number of the independent variable (time) [Default is 0 (first)].");
+	GMT_Usage (API, 1, "\n-T[<min>/<max>/]<inc>[+i|n][+a][+t][+u]");
+	GMT_Usage (API, -2, "Make evenly spaced output time steps from <min> to <max> by <inc>. "
+		"For absolute time resampling, append a valid time unit (%s) to the increment and add +t. "
+		"For spatial resampling with distance computed from the first two columns, specify increment as "
+		"<inc> and append a geospatial distance unit (%s) or c (for Cartesian distances). "
+		"See -A to control how the spatial resampling is done.", GMT_TIME_UNITS_DISPLAY, GMT_LEN_UNITS_DISPLAY);
+	GMT_Usage (API, 3, "+a Add any internally computed distances as a final output column [no distances added].");
+	GMT_Usage (API, 3, "+n Indicate <inc> is the number of t-values to produce over the range instead.");
+	GMT_Usage (API, 3, "+i Indicate <inc> is the reciprocal of desired <inc> (e.g., 3 for 0.3333.....).");
+	GMT_Usage (API, -2, "Alternatively, <inc> is a file with output times in the first column, or a comma-separated list.");
+	GMT_Usage (API, 3, "+u Ensure unique and sorted entries in the array, eliminating duplicates.");
 	GMT_Option (API, "V");
-	GMT_Message (API, GMT_TIME_NONE, "\t-W Give column number of weights for smoothing spline [no weights].\n");
-	GMT_Option (API, "bi2,bo,d,e,f,g,h,i,j,o,q,s,.");
+	GMT_Usage (API, 1, "\n-W<w_col>");
+	GMT_Usage (API, -2, "Give column number of weights for smoothing spline (requires -Fs) [no weights].");
+	GMT_Option (API, "bi2,bo,d,e,f,g,h,i,j,o,q,s,w,.");
 
 	return (GMT_MODULE_USAGE);
 }
@@ -177,30 +190,35 @@ static int parse (struct GMT_CTRL *GMT, struct SAMPLE1D_CTRL *Ctrl, struct GMT_O
 		switch (opt->option) {
 
 			case '<':	/* Skip input files */
-				if (GMT_Get_FilePath (GMT->parent, GMT_IS_DATASET, GMT_IN, GMT_FILE_REMOTE, &(opt->arg))) n_errors++;;
+				if (GMT_Get_FilePath (API, GMT_IS_DATASET, GMT_IN, GMT_FILE_REMOTE, &(opt->arg))) n_errors++;;
 				break;
 			case '>':	/* Got named output file */
 				if (n_files++ > 0) { n_errors++; continue; }
 				Ctrl->Out.active = true;
 				if (opt->arg[0]) Ctrl->Out.file = strdup (opt->arg);
-				if (GMT_Get_FilePath (GMT->parent, GMT_IS_DATASET, GMT_OUT, GMT_FILE_LOCAL, &(Ctrl->Out.file))) n_errors++;
+				if (GMT_Get_FilePath (API, GMT_IS_DATASET, GMT_OUT, GMT_FILE_LOCAL, &(Ctrl->Out.file))) n_errors++;
 				break;
 
 			/* Processes program-specific parameters */
 
 			case 'A':	/* Change track resampling mode */
+				n_errors += gmt_M_repeated_module_option (API, Ctrl->A.active);
 				Ctrl->A.active = true;
-				switch (opt->arg[0]) {
-					case 'f': Ctrl->A.mode = GMT_TRACK_FILL;   break;
-					case 'm': Ctrl->A.mode = GMT_TRACK_FILL_M; break;
-					case 'p': Ctrl->A.mode = GMT_TRACK_FILL_P; break;
-					case 'r': Ctrl->A.mode = GMT_TRACK_SAMPLE_FIX; break;
-					case 'R': Ctrl->A.mode = GMT_TRACK_SAMPLE_ADJ; break;
-					default: GMT_Report (API, GMT_MSG_ERROR, "Option -A: Bad modifier %c\n", opt->arg[0]); n_errors++; break;
+				if (opt->arg[0] != '+') {	/* Gave a mode */
+					switch (opt->arg[0]) {
+						case 'f': Ctrl->A.mode = GMT_TRACK_FILL;   break;
+						case 'm': Ctrl->A.mode = GMT_TRACK_FILL_M; break;
+						case 'p': Ctrl->A.mode = GMT_TRACK_FILL_P; break;
+						case 'r': Ctrl->A.mode = GMT_TRACK_SAMPLE_FIX; break;
+						case 'R': Ctrl->A.mode = GMT_TRACK_SAMPLE_ADJ; break;
+						default: GMT_Report (API, GMT_MSG_ERROR, "Option -A: Bad modifier %c\n", opt->arg[0]); n_errors++; break;
+					}
 				}
-				if (strstr (opt->arg, "+l")) Ctrl->A.loxo = true;
+				if (strstr (opt->arg, "+d")) Ctrl->A.delete = true;
+				if (strstr (opt->arg, "+l")) Ctrl->A.loxo = true;		/* Note: spherical only */
 				break;
 			case 'F':
+				n_errors += gmt_M_repeated_module_option (API, Ctrl->F.active);
 				Ctrl->F.active = true;
 				switch (opt->arg[0]) {
 					case 'l':
@@ -217,7 +235,12 @@ static int parse (struct GMT_CTRL *GMT, struct SAMPLE1D_CTRL *Ctrl, struct GMT_O
 						break;
 					case 's':
 						Ctrl->F.mode = GMT_SPLINE_SMOOTH;
-						Ctrl->F.fit = atof (&opt->arg[1]);
+						if (opt->arg[1])
+							Ctrl->F.fit = atof (&opt->arg[1]);
+						else {
+							GMT_Report (API, GMT_MSG_ERROR, "Option -Fs: No fit parameter given\n");
+							n_errors++;
+						}
 						break;
 					default:
 						GMT_Report (API, GMT_MSG_ERROR, "Option -F: Bad spline selector %c\n", opt->arg[0]);
@@ -273,11 +296,13 @@ static int parse (struct GMT_CTRL *GMT, struct SAMPLE1D_CTRL *Ctrl, struct GMT_O
 					Ctrl->N.col = col;
 				}
 				else {	/* Set output knots */
+					n_errors += gmt_M_repeated_module_option (API, Ctrl->T.active);
 					Ctrl->T.active = true;
 					t_arg = opt->arg;
 				}
 				break;
 			case 'W':
+				n_errors += gmt_M_repeated_module_option (API, Ctrl->W.active);
 				if (opt->arg[0]) {
 					col = atoi (opt->arg);
 					n_errors += gmt_M_check_condition (GMT, col < 0, "Option -W: Column number cannot be negative\n");
@@ -308,11 +333,14 @@ static int parse (struct GMT_CTRL *GMT, struct SAMPLE1D_CTRL *Ctrl, struct GMT_O
 		n_errors += gmt_parse_array (GMT, 'T', t_arg, &(Ctrl->T.T), GMT_ARRAY_TIME | GMT_ARRAY_DIST | GMT_ARRAY_NOMINMAX, Ctrl->N.col);
 	}
 
+	n_errors += gmt_M_check_condition (GMT, Ctrl->A.loxo && GMT->common.j.mode == GMT_GEODESIC,
+	                                   "Option -A+l: Requires spherical calculations so -je cannot be used\n");
 	n_errors += gmt_M_check_condition (GMT, Ctrl->N.active && s_arg, "Specify only one of -N and -S\n");
 	n_errors += gmt_check_binary_io (GMT, (Ctrl->N.col >= 2) ? Ctrl->N.col + 1 : 2);
 	n_errors += gmt_M_check_condition (GMT, n_files > 1, "Only one output destination can be specified\n");
 	n_errors += gmt_M_check_condition (GMT, Ctrl->F.type > 2, "Option -F: Only 1st or 2nd derivatives may be requested\n");
-	n_errors += gmt_M_check_condition (GMT, Ctrl->W.active && Ctrl->F.mode != GMT_SPLINE_SMOOTH, "Option -W: Only available with -Fs<p>\n");
+	n_errors += gmt_M_check_condition (GMT, Ctrl->W.active && Ctrl->F.mode != GMT_SPLINE_SMOOTH,
+	                                   "Option -W: Only available with -Fs<p>\n");
 
 	if (Ctrl->F.mode == GMT_SPLINE_SMOOTH && gmt_M_is_zero (Ctrl->F.fit))	/* Convenience check so -Fs0 is the same as -Fc. Place this hear to allow -W (which will be ignore) */
 		Ctrl->F.mode = GMT_SPLINE_CUBIC;
@@ -364,7 +392,7 @@ EXTERN_MSC int GMT_sample1d (void *V_API, int mode, void *args) {
 	GMT->current.setting.interpolant = Ctrl->F.mode % 10;
 	GMT->current.io.skip_if_NaN[GMT_X] = GMT->current.io.skip_if_NaN[GMT_Y] = false;	/* Turn off default GMT NaN-handling for (x,y) which is not the case here */
 	GMT->current.io.skip_if_NaN[Ctrl->N.col] = true;				/* ... But disallow NaN in "time" column */
-	int_mode = gmt_set_interpolate_mode (GMT, Ctrl->F.mode, Ctrl->F.type);	/* What mode we pass to the interpolator */
+	int_mode = gmt_set_interpolate_mode (GMT, Ctrl->F.mode, Ctrl->F.type);	/* What mode we pass to the interpolater */
 
 	if (Ctrl->T.T.spatial) {
 		if (gmt_M_is_cartesian (GMT, GMT_IN) && Ctrl->A.loxo) {
@@ -372,6 +400,7 @@ EXTERN_MSC int GMT_sample1d (void *V_API, int mode, void *args) {
 			Ctrl->A.loxo = false;
 		}
 		if (Ctrl->A.loxo) GMT->current.map.loxodrome = true;
+		GMT->hidden.sample_along_arc = true;	/* Ensure equidistant sampling along arc, not cord */
 	}
 
 	geometry = (Ctrl->T.T.spatial) ? GMT_IS_LINE : GMT_IS_NONE;
@@ -400,6 +429,15 @@ EXTERN_MSC int GMT_sample1d (void *V_API, int mode, void *args) {
 		GMT_Report (API, GMT_MSG_ERROR, "Requested time column is greater than data number of columns (%d)\n", (int)Din->n_columns);
 		Return (GMT_RUNTIME_ERROR);
 	}
+	if (Ctrl->A.delete) {	/* Remove duplicate rows based on time column */
+		uint64_t tcol = Ctrl->N.col;	/* The single time column */
+		int64_t n_dup = gmt_eliminate_duplicates (API, Din, &tcol, 1, false);
+		if (n_dup < 0) {
+			Return (GMT_RUNTIME_ERROR);
+		}
+		else if (n_dup)
+			GMT_Report (API, GMT_MSG_INFORMATION, "Removed %" PRId64 " records with no change in the time column\n", n_dup);
+	}
 	if (!Ctrl->T.active) {	/* Did not have information for -Tinc during parsing, do now */
 		char string[GMT_LEN32] = {""};
 		/* Get the first data increment as our fixed increment */
@@ -427,7 +465,7 @@ EXTERN_MSC int GMT_sample1d (void *V_API, int mode, void *args) {
 		for (seg = 0; seg < Din->table[tbl]->n_segments; seg++) {
 			S = Din->table[tbl]->segment[seg];	/* Current segment */
 			if (S->n_rows < 2) {
-				GMT_Report (API, GMT_MSG_WARNING, "Segment %" PRIu64 " in table %" PRIu64 " has < 2 records - skipped as no interpolation is possible\n", seg, tbl);
+				GMT_Report (API, GMT_MSG_WARNING, "Table %" PRIu64 " Segment %" PRIu64 " has %" PRIu64 " record - no interpolation possible\n", tbl, seg, S->n_rows);
 				continue;
 			}
 			gmt_M_memset (nan_flag, Din->n_columns, unsigned char);
@@ -456,8 +494,16 @@ EXTERN_MSC int GMT_sample1d (void *V_API, int mode, void *args) {
 			}
 			else {	/* Generate evenly spaced output */
 				double min, max;
-				min = (Ctrl->T.T.delay[GMT_X]) ? ceil (S->data[Ctrl->N.col][0] / Ctrl->T.T.inc) * Ctrl->T.T.inc : Ctrl->T.T.min;
-				max = (Ctrl->T.T.delay[GMT_Y]) ? floor (S->data[Ctrl->N.col][S->n_rows-1] / Ctrl->T.T.inc) * Ctrl->T.T.inc : Ctrl->T.T.max;
+				if (S->data[Ctrl->N.col][0] > S->data[Ctrl->N.col][S->n_rows-1]) {	/* t-column is monotonically decreasing */
+					min = (Ctrl->T.T.delay[GMT_X]) ? floor (S->data[Ctrl->N.col][0] / Ctrl->T.T.inc) * Ctrl->T.T.inc : Ctrl->T.T.min;
+					max = (Ctrl->T.T.delay[GMT_Y]) ? ceil (S->data[Ctrl->N.col][S->n_rows-1] / Ctrl->T.T.inc) * Ctrl->T.T.inc : Ctrl->T.T.max;
+					Ctrl->T.T.reverse = true;	/* Flag we are monotonically decreasing in time for this segment */
+				}
+				else {
+					min = (Ctrl->T.T.delay[GMT_X]) ? ceil (S->data[Ctrl->N.col][0] / Ctrl->T.T.inc) * Ctrl->T.T.inc : Ctrl->T.T.min;
+					max = (Ctrl->T.T.delay[GMT_Y]) ? floor (S->data[Ctrl->N.col][S->n_rows-1] / Ctrl->T.T.inc) * Ctrl->T.T.inc : Ctrl->T.T.max;
+					Ctrl->T.T.reverse = false;	/* Flag we are monotonically increasing in time for this segment */
+				}
 				if (gmt_create_array(GMT, 'T', &(Ctrl->T.T), &min, &max) != GMT_NOERROR) {
 					GMT_Report(API, GMT_MSG_WARNING, "Segment %" PRIu64 " in table %" PRIu64 " had troubles.\n", seg, tbl);
 					continue;
@@ -520,11 +566,13 @@ EXTERN_MSC int GMT_sample1d (void *V_API, int mode, void *args) {
 			Dout->table[tbl]->segment[seg] = Sout;
 		}
 	}
-	if (GMT_Write_Data (API, GMT_IS_DATASET, GMT_IS_FILE, geometry, 0, NULL, Ctrl->Out.file, Dout) != GMT_NOERROR) {
+	if (GMT_Write_Data (API, GMT_IS_DATASET, GMT_IS_FILE, geometry, GMT_WRITE_NORMAL, NULL, Ctrl->Out.file, Dout) != GMT_NOERROR) {
 		Return (API->error);
 	}
 
 	gmt_M_free (GMT, nan_flag);
 
+	GMT->hidden.sample_along_arc = false;	/* Reset */
+
 	Return (GMT_NOERROR);
 }
