@@ -8,14 +8,14 @@
 
 /*!
  * \file segy_io.h
- * \brief 
+ * \brief
  */
 
 #ifndef SEGY_IO_H
 #define SEGY_IO_H
 
 #include "gmt_dev.h"
-#include "common_byteswap.h"
+#include "gmt_common_byteswap.h"
 
 #include "segy.h"
 #include "segyreel.h"
