@@ -1,6 +1,6 @@
 /*--------------------------------------------------------------------
  *
- *   Copyright (c) 1999-2020 by the GMT Team (https://www.generic-mapping-tools.org/team.html)
+ *   Copyright (c) 1999-2021 by the GMT Team (https://www.generic-mapping-tools.org/team.html)
  *
  *   This program is free software; you can redistribute it and/or modify
  *   it under the terms of the GNU Lesser General Public License as published by
@@ -59,12 +59,15 @@ EXTERN_MSC void gmtlib_get_point_from_r_az (struct GMT_CTRL *GMT, double lon0, d
 #define SPOTTER_N_GRID		100
 #define SPOTTER_D_CUT		1.0e-6
 
-void spotter_rot_usage (struct GMTAPI_CTRL *API, char option) {
-	GMT_Message (API, GMT_TIME_NONE, "\t-%c Specify file with the rotations to be used (see documentation for format).\n", option);
-	GMT_Message (API, GMT_TIME_NONE, "\t   Append +i if you want to invert the finite rotations prior to use.\n");
-	GMT_Message (API, GMT_TIME_NONE, "\t   Alternative 1: Give a single rotation as plon/plat/prot.\n");
-	GMT_Message (API, GMT_TIME_NONE, "\t   Alternative 2: Give two plate IDs separated by a hyphen (e.g., PAC-MBL)\n");
-	GMT_Message (API, GMT_TIME_NONE, "\t   to extract that rotation from the GPlates rotation database.\n");
+void spotter_rot_usage (struct GMTAPI_CTRL *API) {
+	GMT_Usage (API, 1, "\n%s", SPOTTER_E_OPT);
+	GMT_Usage (API, -2, "Specify rotation(s) to use in one of three ways:");
+	GMT_Usage (API, 3, "%s Give a file with a series of rotations to be used (see documentation for format).", GMT_LINE_BULLET);
+	GMT_Usage (API, 3, "%s Give a single rotation as <plon>/<plat>/<prot> (in degrees).", GMT_LINE_BULLET);
+	GMT_Usage (API, 3, "%s Give two plate IDs separated by a hyphen (e.g., PAC-MBL) "
+		"to extract that rotation from the GPlates rotation database.", GMT_LINE_BULLET);
+	GMT_Usage (API, -2, "Append +i if you want to invert the finite rotation(s) prior to use.");
+
 }
 
 /* Sort functions used to order the rotations */
@@ -349,6 +352,7 @@ unsigned int spotter_parse (struct GMT_CTRL *GMT, char option, char *arg, struct
 	char txt_a[GMT_LEN256] = {""}, txt_b[GMT_LEN256] = {""}, txt_c[GMT_LEN256] = {""}, *c = NULL;
 	gmt_M_unused(GMT);
 	if ((c = strstr (arg, "+i"))) c[0] = '\0';	/* Chop off modifier */
+	if (c) R->invert = true;	/* Gave +i */
 	if (k == 0 && spotter_GPlates_pair (arg)) {	/* A GPlates plate pair to look up in the rotation table */
 		R->file = strdup (arg);
 		GMT_Report (GMT->parent, GMT_MSG_DEBUG, "Received GPlates pair: %s\n", arg);
@@ -356,12 +360,12 @@ unsigned int spotter_parse (struct GMT_CTRL *GMT, char option, char *arg, struct
 	else if (!gmt_access (GMT, &arg[k], F_OK)) {	/* Was given a file (with possible leading + flag) */
 		R->file = strdup (&arg[k]);
 		if (GMT_Get_FilePath (GMT->parent, GMT_IS_DATASET, GMT_IN, GMT_FILE_REMOTE, &(R->file))) n_errors++;
-		if (k == 1 || c) R->invert = true;
+		if (k == 1) R->invert = true;
 		GMT_Report (GMT->parent, GMT_MSG_DEBUG, "Received rotation file: %s\n", R->file);
 	}
 	else if (gmt_file_is_cache (GMT->parent, arg)) {	/* Was given a remote file */
 		R->file = strdup (&arg[k]);
-		if (k == 1 || c) R->invert = true;
+		if (k == 1) R->invert = true;
 		GMT_Report (GMT->parent, GMT_MSG_DEBUG, "Received rotation file: %s\n", R->file);
 	}
 	else {	/* Apply a fixed total reconstruction rotation to all input points  */
@@ -399,7 +403,7 @@ void spotter_setrot (struct GMT_CTRL *GMT, struct EULER *e) {
 	e->lat_r = e->lat * D2R;
 }
 
-int spotter_init (struct GMT_CTRL *GMT, char *file, struct EULER **p, unsigned int flowline, bool total_out, bool invert, double *t_max) {
+int spotter_init (struct GMT_CTRL *GMT, char *infile, struct EULER **p, unsigned int flowline, bool total_out, bool invert, double *t_max) {
 	/* file;	Name of file with backward stage poles, always GEOCENTRIC */
 	/* p;		Pointer to stage pole array */
 	/* flowline;	1 if flowlines rather than hotspot-tracks are needed */
@@ -414,9 +418,11 @@ int spotter_init (struct GMT_CTRL *GMT, char *file, struct EULER **p, unsigned i
 	FILE *fp = NULL;
 	struct EULER *e = NULL;
 	char buffer[GMT_BUFSIZ] = {""}, A[GMT_LEN64] = {""}, B[GMT_LEN64] = {""}, txt[GMT_LEN64] = {""}, comment[GMT_BUFSIZ] = {""};
-	char Plates[GMT_BUFSIZ] = {""}, Rotations[GMT_BUFSIZ] = {""}, *this_c = NULL;
+	char file[PATH_MAX] = {""}, Plates[GMT_BUFSIZ] = {""}, Rotations[GMT_BUFSIZ] = {""}, *this_c = NULL;
 	double K[9];
 
+	strncpy (file, infile, PATH_MAX);
+	gmt_filename_get (file);	/* Replace any ASCII 29 with spaces */
 	if (gmt_file_is_cache (GMT->parent, file)) {	/* Must be a cache file */
 		gmt_download_file_if_not_found (GMT, file, 0);
 	}
@@ -488,7 +494,9 @@ int spotter_init (struct GMT_CTRL *GMT, char *file, struct EULER **p, unsigned i
 	if (flowline) total_out = true;	/* Override so we get finite poles for conversion to forward stage poles at the end */
 
 	while (gmt_fgets (GMT, buffer, GMT_BUFSIZ, fp) != NULL) { /* Expects lon lat t0 t1 ccw-angle */
-		if (buffer[0] == '#' || buffer[0] == '\n') continue;
+		if (buffer[0] == '#') continue;
+		gmt_chop (buffer);
+		if (gmt_is_a_blank_line (buffer)) continue;
 
 		if (GPlates) {
 			if ((nf = sscanf (buffer, "%d %lf %lf %lf %lf %d %[^\n]", &p1, &t, &lat, &lon, &rot, &p2, comment)) != 7) continue;
@@ -517,6 +525,7 @@ int spotter_init (struct GMT_CTRL *GMT, char *file, struct EULER **p, unsigned i
 			if (nf == 4 || nf == 13) {	/* total reconstruction format: Got lon lat t0 omega [covars], must shift the K's by one */
 				if (i && !total_in) {
 					GMT_Report (GMT->parent, GMT_MSG_ERROR, "Rotation file mixes total reconstruction and stage rotation format\n");
+					gmt_M_free(GMT, e);
 					return -GMT_PARSE_ERROR;
 				}
 				total_in = true;
@@ -630,7 +639,9 @@ int spotter_hotspot_init (struct GMT_CTRL *GMT, char *file, bool geocentric, str
 	e = gmt_M_memory (GMT, NULL, n_alloc, struct HOTSPOT);
 
 	while (gmt_fgets (GMT, buffer, GMT_BUFSIZ, fp) != NULL) {
-		if (buffer[0] == '#' || buffer[0] == '\n') continue;
+		if (buffer[0] == '#') continue;
+		gmt_chop (buffer);
+		if (gmt_is_a_blank_line (buffer)) continue;
 		n = sscanf (buffer, "%lf %lf %s %d %lf %lf %lf %c %c %c %s",
 		            &e[i].lon, &e[i].lat, e[i].abbrev, &ival, &e[i].radius, &e[i].t_off, &e[i].t_on, &create, &fit, &plot, e[i].name);
 		if (n == 3) ival = i + 1;	/* Minimal lon, lat, abbrev */
@@ -1405,9 +1416,9 @@ bool spotter_conf_ellipse (struct GMT_CTRL *GMT, double lon, double lat, double
 
 	/* Find the unique rotation in question */
 
-	for (i = 0, k = -1; k < 0 && i < np; ++i) if (doubleAlmostEqualZero (p[i].t_start, t))
+	for (i = 0, k = GMT_NOTSET; k < 0 && i < np; ++i) if (doubleAlmostEqualZero (p[i].t_start, t))
 		k = i;
-	if (k == -1) return (true);	/* Did not match finite rotation time */
+	if (k == GMT_NOTSET) return (true);	/* Did not match finite rotation time */
 
 	/* Make M(x), the skew-symmetric matrix needed to compute cov of rotated point */
 
