@@ -1,6 +1,6 @@
 /*-----------------------------------------------------------------
  *
- *      Copyright (c) 1999-2020 by the GMT Team (https://www.generic-mapping-tools.org/team.html)
+ *      Copyright (c) 1999-2021 by the GMT Team (https://www.generic-mapping-tools.org/team.html)
  *      See LICENSE.TXT file for copying and redistribution conditions.
  *
  *      This program is free software; you can redistribute it and/or modify
@@ -29,8 +29,6 @@
  *
  */
 
-/* #define DEBUGX */	/* Uncomment for testing */
-
 #include "gmt_dev.h"
 #include "mgd77/mgd77.h"
 #include "x2sys.h"
@@ -156,7 +154,6 @@ GMT_LOCAL double x2syssolve_basis_z (double **P, unsigned int which, uint64_t ro
 	return (P[COL_Z1+which][row]);
 }
 
-
 static void *New_Ctrl (struct GMT_CTRL *GMT) {	/* Allocate and initialize a new control structure */
 	struct X2SYS_SOLVE_CTRL *C;
 
@@ -178,26 +175,31 @@ static void Free_Ctrl (struct GMT_CTRL *GMT, struct X2SYS_SOLVE_CTRL *C) {	/* De
 static int usage (struct GMTAPI_CTRL *API, int level) {
 	const char *name = gmt_show_name_and_purpose (API, THIS_MODULE_LIB, THIS_MODULE_CLASSIC_NAME, THIS_MODULE_PURPOSE);
 	if (level == GMT_MODULE_PURPOSE) return (GMT_NOERROR);
-	GMT_Message (API, GMT_TIME_NONE, "usage: %s -C<column> -Ec|d|g|h|s|t|z -T<TAG> [<coedata>] [%s] [-W[+u]]\n\t[%s] [%s]%s[%s]\n\n",
+	GMT_Usage (API, 0, "usage: %s [<COEdbase>] -C<column> -Ec|d|g|h|s|t|z -T<TAG> [%s] [-W[+u]] [%s] [%s]%s[%s]\n",
 		name, GMT_V_OPT, GMT_bi_OPT, GMT_di_OPT, GMT_x_OPT, GMT_PAR_OPT);
 
 	if (level == GMT_SYNOPSIS) return (GMT_MODULE_SYNOPSIS);
 
-	GMT_Message (API, GMT_TIME_NONE, "\t-C Specify the column name to process (e.g., faa, mag).\n");
-	GMT_Message (API, GMT_TIME_NONE, "\t-E Equation to fit: specify <flag> to indicate model to fit per track:\n");
-	GMT_Message (API, GMT_TIME_NONE, "\t     c (constant offset):   Determine offset [Default].\n");
-	GMT_Message (API, GMT_TIME_NONE, "\t     d (drift by distance): Determine offset and drift-vs-distance rate.\n");
-	GMT_Message (API, GMT_TIME_NONE, "\t     g (gravity latitude):  Determine amplitude of latitude gravity function.\n");
-	GMT_Message (API, GMT_TIME_NONE, "\t     h (magnetic heading):  Determine amplitude of heading magnetic function.\n");
-	GMT_Message (API, GMT_TIME_NONE, "\t     s (data scale):        Determine scaling factor.\n");
-	GMT_Message (API, GMT_TIME_NONE, "\t     t (drift over time):   Determine offset and drift-vs-time rate.\n");
-	GMT_Message (API, GMT_TIME_NONE, "\t     z (data scale/offset): Determine offset and scaling factor.\n");
-	GMT_Message (API, GMT_TIME_NONE, "\t-T <TAG> is the x2sys tag for the data set.\n");
-	GMT_Message (API, GMT_TIME_NONE, "\n\tOPTIONS:\n");
-	GMT_Message (API, GMT_TIME_NONE, "\t<coedata> is the ASCII data output file from x2sys_list [or we read stdin].\n");
+	GMT_Message (API, GMT_TIME_NONE, "  REQUIRED ARGUMENTS:\n");
+	GMT_Usage (API, 1, "\n<COEdbase> File with crossover error data base [standard input].");
+	GMT_Usage (API, 1, "\n-C<column>");
+	GMT_Usage (API, -2, "Specify the column name to process (e.g., faa, mag).");
+	GMT_Usage (API, 1, "\n-Ec|d|g|h|s|t|z");
+	GMT_Usage (API, -2, "Equation to fit: specify <flag> to indicate model to fit per track:");
+	GMT_Usage (API, 3, "c: Constant offset... Determine offset [Default].");
+	GMT_Usage (API, 3, "d: Drift by distance. Determine offset and drift-vs-distance rate.");
+	GMT_Usage (API, 3, "g: Gravity latitude.. Determine amplitude of latitude gravity function.");
+	GMT_Usage (API, 3, "h: Magnetic heading.. Determine amplitude of heading magnetic function.");
+	GMT_Usage (API, 3, "s: Data scale........ Determine scaling factor.");
+	GMT_Usage (API, 3, "t: Drift over time... Determine offset and drift-vs-time rate.");
+	GMT_Usage (API, 3, "z: Data scale/offset. Determine offset and scaling factor.");
+	GMT_Usage (API, 1, "\n-T<TAG>");
+	GMT_Usage (API, -2, "Set the system tag for this compilation.");
+	GMT_Message (API, GMT_TIME_NONE, "\n  OPTIONAL ARGUMENTS:\n");
 	GMT_Option (API, "V");
-	GMT_Message (API, GMT_TIME_NONE, "\t-W Weights are present in last column for weighted fit [no weights].\n");
-	GMT_Message (API, GMT_TIME_NONE, "\t   Append +u to report unweighted mean/std [Default, report weighted stats].\n");
+	GMT_Usage (API, 1, "\n-W[+u]");
+	GMT_Usage (API, -2, "Weights are present in last column for weighted fit [no weights]. "
+		"Append +u to report unweighted mean/std [Default, report weighted stats].");
 	GMT_Option (API, "bi,di,x,.");
 
 	return (GMT_MODULE_USAGE);
@@ -213,6 +215,7 @@ static int parse (struct GMT_CTRL *GMT, struct X2SYS_SOLVE_CTRL *Ctrl, struct GM
 
 	unsigned int n_errors = 0, n_files = 0;
 	struct GMT_OPTION *opt = NULL;
+	struct GMTAPI_CTRL *API = GMT->parent;
 
 	for (opt = options; opt; opt = opt->next) {	/* Process all the options given */
 
@@ -228,10 +231,12 @@ static int parse (struct GMT_CTRL *GMT, struct X2SYS_SOLVE_CTRL *Ctrl, struct GM
 			/* Processes program-specific parameters */
 
 			case 'C':	/* Needed to report correctly */
+				n_errors += gmt_M_repeated_module_option (API, Ctrl->C.active);
 				Ctrl->C.active = true;
 				Ctrl->C.col = strdup (opt->arg);
 				break;
 			case 'E':	/* Which model to fit */
+				n_errors += gmt_M_repeated_module_option (API, Ctrl->E.active);
 				Ctrl->E.active = true;
 				switch (opt->arg[0]) {
 					case 'c':
@@ -256,16 +261,18 @@ static int parse (struct GMT_CTRL *GMT, struct X2SYS_SOLVE_CTRL *Ctrl, struct GM
 						Ctrl->E.mode = F_IS_SCALE_OFF;
 						break;
 					default:
-						GMT_Report (GMT->parent, GMT_MSG_ERROR, "Unrecognized model: %s\n", opt->arg);
+						GMT_Report (API, GMT_MSG_ERROR, "Unrecognized model: %s\n", opt->arg);
 						n_errors++;
 						break;
 				}
 				break;
 			case 'T':
+				n_errors += gmt_M_repeated_module_option (API, Ctrl->T.active);
 				Ctrl->T.active = true;
 				Ctrl->T.TAG = strdup (opt->arg);
 				break;
 			case 'W':
+				n_errors += gmt_M_repeated_module_option (API, Ctrl->W.active);
 				Ctrl->W.active = true;
 				if (!strcmp (opt->arg, "+u") || opt->arg[0] == 'u')		/* Report unweighted statistics anyway */
 					Ctrl->W.unweighted_stats = true;
@@ -283,7 +290,6 @@ static int parse (struct GMT_CTRL *GMT, struct X2SYS_SOLVE_CTRL *Ctrl, struct GM
 	return (n_errors ? GMT_PARSE_ERROR : GMT_NOERROR);
 }
 
-
 #define bailout(code) {gmt_M_free_options (mode); return (code);}
 #define Return(code) {Free_Ctrl (GMT, Ctrl); gmt_end_module (GMT, GMT_cpy); bailout (code);}
 
@@ -339,13 +345,17 @@ EXTERN_MSC int GMT_x2sys_solve (void *V_API, int mode, void *args) {
 
 	/* Initialize system via the tag */
 
-	if (x2sys_err_fail (GMT, x2sys_set_system (GMT, Ctrl->T.TAG, &S, &B, &GMT->current.io), Ctrl->T.TAG))
+	if (x2sys_err_fail (GMT, x2sys_set_system (GMT, Ctrl->T.TAG, &S, &B, &GMT->current.io), Ctrl->T.TAG)) {
+		x2sys_end (GMT, S);
 		Return (GMT_RUNTIME_ERROR);
+	}
 
 	/* Verify that the chosen column is known to the system */
 
-	if (Ctrl->C.col && x2sys_err_fail (GMT, x2sys_pick_fields (GMT, Ctrl->C.col, S), "-C"))
+	if (Ctrl->C.col && x2sys_err_fail (GMT, x2sys_pick_fields (GMT, Ctrl->C.col, S), "-C")) {
+		x2sys_end (GMT, S);
 		Return (GMT_RUNTIME_ERROR);
+	}
 	if (S->n_out_columns != 1) {
 		GMT_Report (API, GMT_MSG_ERROR, "Option -C must specify a single column name\n");
 		x2sys_end (GMT, S);
@@ -430,7 +440,7 @@ EXTERN_MSC int GMT_x2sys_solve (void *V_API, int mode, void *args) {
 	}
 
 	for (k = 0; k < bin_expect; k++)	/* All input columns are floating point numbers here */
-		gmt_set_column (GMT, GMT_IN, k, GMT_IS_FLOAT);
+		gmt_set_column_type (GMT, GMT_IN, k, GMT_IS_FLOAT);
 
 	/* Open the crossover info */
 
@@ -596,6 +606,7 @@ EXTERN_MSC int GMT_x2sys_solve (void *V_API, int mode, void *args) {
 
 	normalize = (Ctrl->E.mode == F_IS_DRIFT_T || Ctrl->E.mode == F_IS_DRIFT_D);	/* Only when the linear drift term is in effect */
 	if (normalize) {	/* For numerical stability, normalize distances or times to fall in 0-1 range */
+		char *type = (Ctrl->E.mode == F_IS_DRIFT_T) ? "times" : "distances";
 		min_extent = DBL_MAX;	max_extent = -DBL_MAX;
 		j = (Ctrl->E.mode == F_IS_DRIFT_T) ? COL_T1 : COL_D1;	/* Which variable we are working on */
 		for (k = 0; k < n_COE; k++) {
@@ -606,6 +617,7 @@ EXTERN_MSC int GMT_x2sys_solve (void *V_API, int mode, void *args) {
 		}
 		range = max_extent - min_extent;
 		for (k = 0; k < n_COE; k++) for (i = 0; i < 2; i++) data[j+i][k] /= range;	/* Get normalized time or distance */
+		GMT_Report (API, GMT_MSG_INFORMATION, "Normalized all %s by dividing with the maximum range = %g\n", type, range);
 	}
 
 	/* Estimate old weighted mean and std.dev */
@@ -711,7 +723,7 @@ EXTERN_MSC int GMT_x2sys_solve (void *V_API, int mode, void *args) {
 				}
 			}
 			else
-				GMT_Report (API, GMT_MSG_INFORMATION, "%" PRIu64 " tracks form a single connected cluster\n", n);
+				GMT_Report (API, GMT_MSG_INFORMATION, "%" PRIu64 " tracks form a single connected cluster\n", n_tracks);
 		}
 	}
 
@@ -759,12 +771,15 @@ EXTERN_MSC int GMT_x2sys_solve (void *V_API, int mode, void *args) {
 
 	GMT_Report (API, GMT_MSG_INFORMATION, "Matrix equation N * a = b: (N = %" PRIu64 " x %" PRIu64 ")\n", m, m);
 
-#ifdef DEBUGX
-	for (i = 0; i < m; i++) {
-		for (j = 0; j < m; j++) GMT_Message (API, GMT_TIME_NONE, "%8.2f\t", N[i*m+j]);
-		GMT_Message (API, GMT_TIME_NONE, "\t%8.2f\n", b[i]);
+	if (gmt_M_is_verbose (GMT, GMT_MSG_DEBUG)) {
+		char format1[GMT_LEN64] = {""}, format2[GMT_LEN64] = {""};
+		snprintf (format1, GMT_LEN64-1, "%s\t", GMT->current.setting.format_float_out);
+		snprintf (format2, GMT_LEN64-2, "\t%s\n", GMT->current.setting.format_float_out);
+		for (i = 0; i < m; i++) {
+			for (j = 0; j < m; j++) GMT_Message (API, GMT_TIME_NONE, format1, N[i*m+j]);
+			GMT_Message (API, GMT_TIME_NONE, format2, b[i]);
+		}
 	}
-#endif
 
 	/* Get LS solution */
 
@@ -798,9 +813,7 @@ EXTERN_MSC int GMT_x2sys_solve (void *V_API, int mode, void *args) {
 			Sx += (data[COL_WW][k] * e_k);
 			Sxx += (data[COL_WW][k] * e_k * e_k);
 		}
-#ifdef DEBUGX
-		GMT_Message (API, GMT_TIME_NONE, "COE # %d: Before: %g After: %g\n", k, data[COL_COE][k], e_k);
-#endif
+		GMT_Report (API, GMT_MSG_DEBUG, "COE # %d: Before: %g After: %g\n", k, data[COL_COE][k], e_k);
 	}
 	if (Ctrl->W.unweighted_stats) Sw = (double)n_COE;
 	new_mean = Sx / Sw;
